Package: formattable
Title: Create 'Formattable' Data Structures
Version: 0.2.1
Authors@R: c(
    person("Kun", "Ren", role = c("aut", "cre"), email = "ken@renkun.me"),
    person("Kenton", "Russell", role = "aut", email = "kent.russell@timelyportfolio.com")
    )
Description: Provides functions to create formattable vectors and data frames.
    'Formattable' vectors are printed with text formatting, and formattable
    data frames are printed with multiple types of formatting in HTML
    to improve the readability of data presented in tabular form rendered in
    web pages.
Depends: R (> 3.0.2)
Date: 2021-01-05
Imports: methods, htmltools, htmlwidgets, knitr, rmarkdown
License: MIT + file LICENSE
LazyData: true
URL: https://renkun-ken.github.io/formattable/,
        https://github.com/renkun-ken/formattable
BugReports: https://github.com/renkun-ken/formattable/issues
ByteCompile: TRUE
Suggests: testthat, DT, shiny, covr
RoxygenNote: 7.1.1
VignetteBuilder: knitr
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2021-01-07 01:43:38 UTC; ken
Author: Kun Ren [aut, cre],
  Kenton Russell [aut]
Maintainer: Kun Ren <ken@renkun.me>
Repository: CRAN
Date/Publication: 2021-01-07 20:20:10 UTC

# Generated by roxygen2: do not edit by hand

S3method("%%",formattable)
S3method("&",formattable)
S3method("*",formattable)
S3method("+",formattable)
S3method("-",formattable)
S3method("/",formattable)
S3method("[",formattable)
S3method("[<-",formattable)
S3method("[[",formattable)
S3method("[[<-",formattable)
S3method("|",formattable)
S3method(accounting,character)
S3method(accounting,default)
S3method(all,formattable)
S3method(any,formattable)
S3method(as.character,formattable)
S3method(as.datatable,formattable)
S3method(as.htmlwidget,formattable)
S3method(as.list,formattable)
S3method(c,formattable)
S3method(comma,character)
S3method(comma,default)
S3method(csscolor,character)
S3method(csscolor,matrix)
S3method(cummax,formattable)
S3method(cummin,formattable)
S3method(cumsum,formattable)
S3method(currency,character)
S3method(currency,default)
S3method(diff,formattable)
S3method(format,formattable)
S3method(formattable,Date)
S3method(formattable,POSIXct)
S3method(formattable,POSIXlt)
S3method(formattable,data.frame)
S3method(formattable,default)
S3method(formattable,factor)
S3method(formattable,formattable)
S3method(formattable,logical)
S3method(formattable,numeric)
S3method(formattable,table)
S3method(knit_print,formattable)
S3method(max,formattable)
S3method(mean,formattable)
S3method(median,formattable)
S3method(min,formattable)
S3method(percent,character)
S3method(percent,default)
S3method(print,formattable)
S3method(print,formatter)
S3method(quantile,formattable)
S3method(rep,formattable)
S3method(sum,formattable)
S3method(unique,formattable)
export(accounting)
export(area)
export(as.datatable)
export(as.htmlwidget)
export(color_bar)
export(color_text)
export(color_tile)
export(comma)
export(csscolor)
export(currency)
export(digits)
export(format_table)
export(formattable)
export(formattableOutput)
export(formatter)
export(gradient)
export(icontext)
export(is.formattable)
export(normalize)
export(normalize_bar)
export(percent)
export(prefix)
export(proportion)
export(proportion_bar)
export(qrank)
export(renderFormattable)
export(scientific)
export(style)
export(suffix)
export(vmap)
importFrom(grDevices,col2rgb)
importFrom(htmltools,attachDependencies)
importFrom(htmltools,doRenderTags)
importFrom(htmltools,tag)
importFrom(htmltools,tagList)
importFrom(htmltools,tags)
importFrom(htmlwidgets,createWidget)
importFrom(htmlwidgets,shinyRenderWidget)
importFrom(htmlwidgets,shinyWidgetOutput)
importFrom(knitr,asis_output)
importFrom(knitr,kable)
importFrom(knitr,knit_print)
importFrom(rmarkdown,html_dependency_bootstrap)
importFrom(rmarkdown,html_dependency_jquery)
importFrom(stats,median)
importFrom(stats,quantile)

# formattable

<!-- badges: start -->
[![R build status](https://github.com/renkun-ken/formattable/workflows/rcc/badge.svg)](https://github.com/renkun-ken/formattable/actions)
[![codecov.io](https://codecov.io/github/renkun-ken/formattable/coverage.svg?branch=master)](https://codecov.io/github/renkun-ken/formattable?branch=master)
[![CRAN Version](https://www.r-pkg.org/badges/version/formattable)](https://cran.r-project.org/package=formattable)
<!-- badges: end -->

This package is designed for applying formatting on vectors and data frames to make data presentation easier, richer, more flexible and hopefully convey more information.

This document is also translated into [日本語](https://github.com/renkun-ken/formattable/blob/master/README.ja.md) by [@hoxo_m](https://github.com/hoxo-m), [@dichika](https://github.com/dichika) and [@teramonagi](https://github.com/teramonagi).

## Install

The package is available on both GitHub and CRAN.

Install from GitHub:

```r
# install.packages("devtools")
devtools::install_github("renkun-ken/formattable")
```

Install from CRAN:

```r
install.packages("formattable")
```

## Introduction

Atomic vectors are basic units to store data. Some data can be read more easily with formatting. A numeric vector, for example, stores a group of percentage numbers yet still shows in the form of typical floating numbers. This package provides functions to create data structures with predefined formatting rules so that these objects store the original data but are printed with formatting.

The package provides several typical formattable objects such as `percent`, `comma`, `currency`, `accounting` and `scientific`. These objects are essentially numeric vectors with pre-defined formatting rules and parameters. For example,


```r
library(formattable)
p <- percent(c(0.1, 0.02, 0.03, 0.12))
p
```

```
## [1] 10.00% 2.00%  3.00%  12.00%
```

The percent vector is no different from a numeric vector but has a percentage representation as being
printed. It works with arithmetic operations and common functions and preserves its formatting.


```r
p + 0.05
```

```
## [1] 15.00% 7.00%  8.00%  17.00%
```

```r
max(p)
```

```
## [1] 12.00%
```


```r
balance <- accounting(c(1000, 500, 200, -150, 0, 1200))
balance
```

```
## [1] 1,000.00 500.00   200.00   (150.00) 0.00     1,200.00
```

```r
balance + 1000
```

```
## [1] 2,000.00 1,500.00 1,200.00 850.00   1,000.00 2,200.00
```

These functions are special cases of what `formattable()` can do. `formattable()` applies highly customizable formatting to objects of a wide range of classes like `numeric`, `logical`, `factor`, `Date`, `data.frame`, etc. A typical data frame may look more friendly with `formattable` column vectors. For example,


```r
p <- data.frame(
  id = c(1, 2, 3, 4, 5), 
  name = c("A1", "A2", "B1", "B2", "C1"),
  balance = accounting(c(52500, 36150, 25000, 18300, 7600), format = "d"),
  growth = percent(c(0.3, 0.3, 0.1, 0.15, 0.15), format = "d"),
  ready = formattable(c(TRUE, TRUE, FALSE, FALSE, TRUE), "yes", "no"))
p
```

```
##   id name balance growth ready
## 1  1   A1  52,500    30%   yes
## 2  2   A2  36,150    30%   yes
## 3  3   B1  25,000    10%    no
## 4  4   B2  18,300    15%    no
## 5  5   C1   7,600    15%   yes
```

## Formatting tables in dynamic document

In a typical workflow of dynamic document production, [knitr](https://yihui.org/knitr/) and [rmarkdown](https://rmarkdown.rstudio.com/) are powerful tools to render documents with R code to different types of portable documents.

knitr is able to render an RMarkdown document (markdown document with R code chunks) to Markdown document. rmarkdown calls [pandoc](https://johnmacfarlane.net/pandoc) to render a markdown document to HTML web page. To put a table (`data.frame` in R) on the page, one may call `knitr::kable` to produce its markdown representation. By default the resulted table is in a plain theme with no additional formatting. However, in some cases, additional formatting may help clarify the information and make contrast of the data. This package provides functions to produce formatted tables in dynamic documents.


```r
df <- data.frame(
  id = 1:10,
  name = c("Bob", "Ashley", "James", "David", "Jenny", 
    "Hans", "Leo", "John", "Emily", "Lee"), 
  age = c(28, 27, 30, 28, 29, 29, 27, 27, 31, 30),
  grade = c("C", "A", "A", "C", "B", "B", "B", "A", "C", "C"),
  test1_score = c(8.9, 9.5, 9.6, 8.9, 9.1, 9.3, 9.3, 9.9, 8.5, 8.6),
  test2_score = c(9.1, 9.1, 9.2, 9.1, 8.9, 8.5, 9.2, 9.3, 9.1, 8.8),
  final_score = c(9, 9.3, 9.4, 9, 9, 8.9, 9.25, 9.6, 8.8, 8.7),
  registered = c(TRUE, FALSE, TRUE, FALSE, TRUE, TRUE, TRUE, FALSE, FALSE, FALSE),
  stringsAsFactors = FALSE)
```

Plain table:


| id|name   | age|grade | test1_score| test2_score| final_score|registered |
|--:|:------|---:|:-----|-----------:|-----------:|-----------:|:----------|
|  1|Bob    |  28|C     |         8.9|         9.1|        9.00|TRUE       |
|  2|Ashley |  27|A     |         9.5|         9.1|        9.30|FALSE      |
|  3|James  |  30|A     |         9.6|         9.2|        9.40|TRUE       |
|  4|David  |  28|C     |         8.9|         9.1|        9.00|FALSE      |
|  5|Jenny  |  29|B     |         9.1|         8.9|        9.00|TRUE       |
|  6|Hans   |  29|B     |         9.3|         8.5|        8.90|TRUE       |
|  7|Leo    |  27|B     |         9.3|         9.2|        9.25|TRUE       |
|  8|John   |  27|A     |         9.9|         9.3|        9.60|FALSE      |
|  9|Emily  |  31|C     |         8.5|         9.1|        8.80|FALSE      |
| 10|Lee    |  30|C     |         8.6|         8.8|        8.70|FALSE      |

Formatted table with the following visualizations:

* Ages are rendered in gradient.
* All A grades are displayed in green bold.
* `test1_score` and `test2_score` are indicated by horizontal bars and are background-colorized: white (low score) to pink (high score)
* `final_score` shows score and ranking. Top 3 are green, and others are gray.
* `registered` texts are transformed to an icon and yes/no text.


```r
library(formattable)

formattable(df, list(
  age = color_tile("white", "orange"),
  grade = formatter("span", style = x ~ ifelse(x == "A", 
    style(color = "green", font.weight = "bold"), NA)),
  area(col = c(test1_score, test2_score)) ~ normalize_bar("pink", 0.2),
  final_score = formatter("span",
    style = x ~ style(color = ifelse(rank(-x) <= 3, "green", "gray")),
    x ~ sprintf("%.2f (rank: %02d)", x, rank(-x))),
  registered = formatter("span",
    style = x ~ style(color = ifelse(x, "green", "red")),
    x ~ icontext(ifelse(x, "ok", "remove"), ifelse(x, "Yes", "No")))
))
```

![formattable](formattable.png)

**The icon set used in the table is by [GLYPHICONS.com](https://GLYPHICONS.com) and included in [Bootstrap](https://getbootstrap.com/docs/3.4/components/#glyphicons).**

## `htmlwidget` conversion in interactive environments

`formattable` will automatically convert to an `htmlwidget` when in an `interactive()` context such as the console or RStudio IDE.  If you would like to avoid this conversion and see the `html` table output, please use `format_table` that calls `knitr::kable` with formatters or call `format` with the `formattable data.frame` object.

## License

This package is under [MIT License](https://opensource.org/licenses/MIT).

#' Create a formatter function making HTML elements
#'
#' @details
#' This function creates a \code{formatter} object which is essentially a
#' closure taking a value and optionally the dataset behind.
#'
#' The formatter produces a character vector of HTML elements represented
#' as strings. The tag name of the elements are specified by \code{.tag},
#' and its attributes are calculated with the given functions or formulas
#' specified in \code{...} given the input vector and/or dataset in behind.
#'
#' Formula like \code{x ~ expr} will behave like \code{function(x) expr}.
#' Formula like \code{~expr} will be evaluated in different manner: \code{expr}
#' will be evaluated in the data frame with the enclosing environment being
#' the formula environment. If a column is formatted according to multiple
#' other columns, \code{~expr} should be used and the column names can directly
#' appear in \code{expr}.
#' @importFrom htmltools tag doRenderTags
#' @param .tag HTML tag name. Uses \code{span} by default.
#' @param ... functions to create attributes of HTML element from data colums.
#' The unnamed element will serve as the function to produce the inner text of the
#' element. If no unnamed element is provided, \code{identity} function will be used
#' to preserve the string representation of the colum values. Function and formula are
#' accepted. See details for how different forms of formula will behave differently.
#' @return a function that transforms a column of data (usually an atomic vector)
#' to formatted data represented in HTML and CSS.
#' @examples
#' top10red <- formatter("span",
#'   style = x ~ ifelse(rank(-x) <= 10, "color:red", NA))
#' yesno <- function(x) ifelse(x, "yes", "no")
#' formattable(mtcars, list(mpg = top10red, qsec = top10red, am = yesno))
#'
#' # format one column by other two columns
#' # make cyl red for records with both mpg and disp rank <= 20
#' f1 <- formatter("span",
#'   style = ~ ifelse(rank(-mpg) <= 20 & rank(-disp) <= 20, "color:red", NA))
#' formattable(mtcars, list(cyl = f1))
#' @export
formatter <- function(.tag, ...) {
  fcall <- match.call(expand.dots = TRUE)
  args <- list(...)

  use_text <- length(args) == 0L ||
    (!is.null(argnames <- names(args)) && all(nzchar(argnames)))

  # create a closure for formattable to build output string
  structure(function(x, data = NULL, text = format(x, trim = TRUE)) {
    if (length(x) == 0L && length(data) == 0L) return(character())
    values <- c(lapply(args, function(arg) {
      value <- if (is.function(arg)) arg(x)
      else if (inherits(arg, "formula")) eval_formula(arg, x, data)
      else arg
      if (is.null(value)) NA else value
    }), if (use_text) list(text))
    tags <- if (length(x) == 1L) {
      list(tag(.tag, values[!is.na(values) & nzchar(values)]))
    } else {
      .mapply(function(...) {
        attrs <- list(...)
        tag(.tag, attrs[!is.na(attrs) & nzchar(attrs)])
      }, values, NULL)
    }
    copy_dim(x, vapply(tags, doRenderTags, character(1L)))
  }, class = c("formatter", "function"))
}

#' @export
print.formatter <- function(x, ...) {
  env <- environment(x)
  print(env$fcall, ...)
  invisible(x)
}

#' Create an area to apply formatter
#'
#' Create an representation of two-dimenstional area
#' to apply formatter function. The area can be one or
#' more columns, one or more rows, or an area of rows
#' and columns.
#'
#' @details
#' The function creates an \code{area} object to store
#' the representation of row and column selector expressions.
#' When the function is called, the expressions and environment
#' of \code{row} and \code{column} are captured for
#' \code{format_table} to evaluate within the context of the
#' input \code{data.frame}, that is, \code{rownames} and
#' \code{colnames} are defined in the context to be the indices
#' of rows and columns, respectively. Therefore, the row names
#' and column names are avaiable symbols when \code{row}
#' and \code{col} are evaluated, respectively, which makes it
#' easier to specify range with names, for example,
#' \code{area(row = row1:row10, col = col1:col5)}.
#'
#' @param row an expression of row range. If missing,
#' \code{TRUE} is used instead.
#' @param col an expression of column range. If missing,
#' \code{TRUE} is used instead.
#' @export
#' @examples
#' area(col = c("mpg", "cyl"))
#' area(col = mpg:cyl)
#' area(row = 1)
#' area(row = 1:10, col = 5:10)
#' area(1:10, col1:col5)
#' @seealso \link{format_table}, \link{formattable.data.frame}
area <- function(row, col) {
  structure(list(
    row = if (missing(row)) TRUE else substitute(row),
    col = if (missing(col)) TRUE else substitute(col),
    envir = parent.frame()),
    class = "area")
}

#' Create a color-tile formatter
#'
#' @param ... parameters passed to \code{gradient}.
#' @export
#' @examples
#' formattable(mtcars, list(mpg = color_tile("white", "pink")))
color_tile <- function(...) {
  formatter("span",
    style = function(x) style(
      display = "block",
      padding = "0 4px",
      "border-radius" = "4px",
      "background-color" = csscolor(gradient(as.numeric(x), ...))))
}

#' Create a color-bar formatter
#'
#' @param color the background color of the bars
#' @param fun the transform function that maps the input vector to
#' values from 0 to 1. Uses \code{proportion} by default.
#' @param ... additional parameters passed to \code{fun}
#' @export
#' @examples
#' formattable(mtcars, list(mpg = color_bar("lightgray", proportion)))
#' @seealso
#' \link{normalize_bar}, \link{proportion_bar}
color_bar <- function(color = "lightgray", fun = "proportion", ...) {
  fun <- match.fun(fun)
  formatter("span",
    style = function(x) style(
      display = "inline-block",
      direction = "rtl",
      "unicode-bidi" = "plaintext",
      "border-radius" = "4px",
      "padding-right" = "2px",
      "background-color" = csscolor(color),
      width = percent(fun(as.numeric(x), ...))
    ))
}

#' Create a color-bar formatter using normalize
#'
#' @param color the background color of the bars
#' @param ... additional parameters passed to \code{normalize}
#' @export
#' @examples
#' formattable(mtcars, list(mpg = normalize_bar()))
#' @seealso
#' \link{color_bar}, \link{normalize}
normalize_bar <- function(color = "lightgray", ...) {
  color_bar(color = color, fun = normalize, ...)
}

#' Create a color-bar formatter using proportion
#'
#' @param color the background color of the bars
#' @param ... additional parameters passed to \code{proportion}
#' @export
#' @examples
#' formattable(mtcars, list(mpg = proportion_bar()))
#' @seealso
#' \link{color_bar}, \link{proportion}
proportion_bar <- function(color = "lightgray", ...) {
  color_bar(color = color, fun = proportion, ...)
}

#' Create a color-text formatter
#'
#' @param ... parameters passed to \code{gradient}.
#' @export
#' @examples
#' formattable(mtcars, list(mpg = color_text("black", "red")))
color_text <- function(...) {
  formatter("span",
    style = function(x) style(
      color = csscolor(gradient(as.numeric(x), ...))))
}

context("htmlwidget")

test_that("conversion", {
  expect_is(as.htmlwidget(formattable(mtcars)), c("formattable_widget", "htmlwidget"))
})

test_that("shiny", {
  # some preliminary testing for shiny functionality
  expect_is(renderFormattable(formattable(head(mtcars))), c("shiny.render.function", "function"))
  expect_is(formattableOutput(0), c("shiny.tag.list", "list"))
})

test_that("as.datatable", {
  f <- formattable(mtcars, list(
    mpg = color_tile("transparent", "lightgray"),
    cyl = color_bar("gray"),
    area(col = vs:carb) ~ formatter("span", style = x ~ style(color = ifelse(x > 0, "red", NA)))))
  dt <- as.datatable(f)
  expect_is(dt, c("datatables", "htmlwidget"))
})

context("formatter")

test_that("formatter", {
  expect_is(formatter("span", style = "color:red"), "function")
  expect_equal(formatter("span", style = "color: red")(c(1, 2, 3)),
    sprintf('<span style="color: red">%s</span>', c(1, 2, 3)))
  expect_equal(formatter("span", function(x) ifelse(x, "yes", "no"))(c(1, 0, 0, 1)),
    paste0("<span>", ifelse(c(1, 0, 0, 1), "yes", "no"), "</span>"))
  expect_equal(formatter("span", x ~ ifelse(x, "yes", "no"))(1),
    paste0("<span>", ifelse(1, "yes", "no"), "</span>"))
  expect_match(capture.output(print(formatter("span"))), "formatter")
  expect_equal(local({
    yes_string <- "YES"
    no_string <- "NO"
    formatter("span", x ~ ifelse(x, yes_string, no_string))(c(TRUE, FALSE))
  }), paste0("<span>", c("YES", "NO"), "</span>"))

  # render text
  bold <- formatter("span", style = "font-weight: bold")
  expect_equal(bold(c(0.1, 0.2)),
    c("<span style=\"font-weight: bold\">0.1</span>",
      "<span style=\"font-weight: bold\">0.2</span>"))
  expect_equal(bold(percent(c(0.1, 0.2))),
    c("<span style=\"font-weight: bold\">10.00%</span>",
      "<span style=\"font-weight: bold\">20.00%</span>"))

  bold_percent <- formatter("span", style = "font-weight: bold", percent)
  expect_equal(bold_percent(c(0.1, 0.2)),
    c("<span style=\"font-weight: bold\">10.00%</span>",
      "<span style=\"font-weight: bold\">20.00%</span>"))

  # dynamic scoping of formula
  expect_equal(local({
    yes <- "yes"
    no <- "no"
    formatter("span", x ~ ifelse(x, yes, no))(c(1, 0, 0, 1))
  }), paste0("<span>", ifelse(c(1, 0, 0, 1), "yes", "no"), "</span>"))

  df <- data.frame(x = c(-1, 0, 1), y = c(-1, 0, 1))
  expect_equal(formatter("span", ~ ifelse(x >= 0 & y >= 0, "yes", "no"))(NULL, df),
    paste0("<span>", ifelse(df$x >= 0 & df$y >= 0, "yes", "no"), "</span>"))
})

test_that("area", {
  expect_is(area(), "area")
  expect_that(area(), is.list)
  expect_identical(area()$row, TRUE)
  expect_identical(area()$col, TRUE)

  a1 <- area(1:10, 1:3)
  expect_that(a1$row, is.language)
  expect_that(a1$col, is.language)
  expect_identical(a1$envir, environment())
})

test_that("formatters", {
  x <- c(0.1, 0.2, 0.3)

  f <- color_tile("white", "pink")
  expect_equal(f(0.1),
    "<span style=\"display: block; padding: 0 4px; border-radius: 4px; background-color: #ffc0cb\">0.1</span>")
  expect_equal(f(x), c(
    "<span style=\"display: block; padding: 0 4px; border-radius: 4px; background-color: #ffffff\">0.1</span>",
    "<span style=\"display: block; padding: 0 4px; border-radius: 4px; background-color: #ffdfe5\">0.2</span>",
    "<span style=\"display: block; padding: 0 4px; border-radius: 4px; background-color: #ffc0cb\">0.3</span>"
  ))
  expect_equal(f(percent(x)), c(
    "<span style=\"display: block; padding: 0 4px; border-radius: 4px; background-color: #ffffff\">10.00%</span>",
    "<span style=\"display: block; padding: 0 4px; border-radius: 4px; background-color: #ffdfe5\">20.00%</span>",
    "<span style=\"display: block; padding: 0 4px; border-radius: 4px; background-color: #ffc0cb\">30.00%</span>"
  ))
})

#' Generic function to create an htmlwidget
#'
#' This function is a generic function to create an \code{htmlwidget}
#' to allow HTML/JS from R in multiple contexts.
#'
#' @param x an object.
#' @param ... arguments to be passed to methods.
#' @export
#' @return a \code{htmlwidget} object
as.htmlwidget <- function(x, ...)
  UseMethod("as.htmlwidget")


#' Convert formattable to an htmlwidget
#'
#' formattable was originally designed to work in \code{rmarkdown} environments.
#' Conversion of a formattable to a htmlwidget will allow use in other contexts
#' such as console, RStudio Viewer, and Shiny.
#'
#' @param x a \code{formattable} object to convert
#' @param width a valid \code{CSS} width
#' @param height a valid \code{CSS} height
#' @param ... reserved for more parameters
#' @return a \code{htmlwidget} object
#'
#' @examples
#' \dontrun{
#' library(formattable)
#' # mtcars (mpg background in gradient: the higher, the redder)
#' as.htmlwidget(
#'   formattable(mtcars, list(mpg = formatter("span",
#'    style = x ~ style(display = "block",
#'    "border-radius" = "4px",
#'    "padding-right" = "4px",
#'    color = "white",
#'    "background-color" = rgb(x/max(x), 0, 0))))
#'   )
#' )
#'
#' # since an htmlwidget, composes well with other tags
#' library(htmltools)
#'
#' browsable(
#'   tagList(
#'     tags$div( class="jumbotron"
#'               ,tags$h1( class = "text-center"
#'                         ,tags$span(class = "glyphicon glyphicon-fire")
#'                         ,"experimental as.htmlwidget at work"
#'               )
#'     )
#'     ,tags$div( class = "row"
#'                ,tags$div( class = "col-sm-2"
#'                           ,tags$p(class="bg-primary", "Hi, I am formattable htmlwidget.")
#'                )
#'                ,tags$div( class = "col-sm-6"
#'                           ,as.htmlwidget( formattable( mtcars ) )
#'                )
#'     )
#'   )
#' )
#' }
#' @importFrom htmlwidgets createWidget
#' @export
as.htmlwidget.formattable <- function(x, width = "100%", height = NULL, ...) {
  if (!is.formattable(x)) stop("expect formattable to be a formattable", call. = FALSE)
  html <- gsub('th align="', 'th class="text-',
    format(x, format = list(format = "html")), fixed = TRUE)

  # forward options using x
  x <- list(html = html)

  # create widget
  createWidget("formattable_widget", x, width = width,
    height = height, package = "formattable", ...)
}

#' @importFrom htmltools tags attachDependencies
#' @importFrom rmarkdown html_dependency_jquery html_dependency_bootstrap
formattable_widget_html <- function(name, package, id, style, class, width, height) {
  attachDependencies(
    tags$div(id = id, class = class, style = style,
      width = width, height = height),
    list(
      html_dependency_jquery(),
      html_dependency_bootstrap("default")
    )
  )
}

#' Widget output function for use in Shiny
#' @param outputId output variable to read from
#' @param width a valid \code{CSS} width or a number
#' @param height valid \code{CSS} height or a number
#' @importFrom htmlwidgets shinyWidgetOutput
#' @export
formattableOutput <- function(outputId, width = "100%", height = "0") {
  shinyWidgetOutput(outputId, "formattable_widget", width, height, package = "formattable")
}

#' Widget render function for use in Shiny
#' @param expr an expression that generates a valid \code{formattable} object
#' @param env the environment in which to evaluate expr.
#' @param quoted is expr a quoted expression (with quote())?
#' This is useful if you want to save an expression in a variable.
#' @importFrom htmlwidgets shinyRenderWidget
#' @export
renderFormattable <- function(expr, env = parent.frame(), quoted = FALSE) {
  if (!quoted) { expr <- substitute(formattable::as.htmlwidget(expr)) } # force quoted
  shinyRenderWidget(expr, formattableOutput, env, quoted = TRUE)
}

base_ifelse <- getExportedValue("base", "ifelse")

as_numeric <- function(x) if (is.numeric(x)) x else as.numeric(x)

set_class <- function(x, class) {
  if (!inherits(x, class))
    class(x) <- c(class, class(x))
  x
}

create_obj <- function(x, class, attributes) {
  if (!missing(attributes)) attr(x, class) <- attributes
  set_class(x, class)
}

reset_class <- function(src, target, class) {
  if (storage.mode(target) == storage.mode(src)) target
  else set_class(unclass(target), class)
}

ifelse <- function(test, yes, no, ...) {
  base_ifelse(test, yes, no)
}

remove_class <- function(x, class) {
  cls <- class(x)
  class(x) <- cls[cls != class]
  x
}

remove_attribute <- function(x, which) {
  attr(x, which) <- NULL
  x
}

copy_obj <- function(src, target, class) {
  create_obj(target, class, attr(src, class, exact = TRUE))
}

fcreate_obj <- function(f, class, x, ...) {
  create_obj(f(remove_class(x, class), ...), class, attr(x, class, exact = TRUE))
}

cop_create_obj <- function(op, class, x, y) {
  if (inherits(x, class)) {
    if (missing(y))
      create_obj(op(remove_class(x, class)), class, attr(x, class, exact = TRUE))
    else
      create_obj(op(remove_class(x, class), unclass(y)), class, attr(x, class, exact = TRUE))
  } else if (inherits(y, class)) {
    create_obj(op(unclass(x), remove_class(y, class)), class, attr(y, class, exact = TRUE))
  } else {
    create_obj(op(x, y), class, NULL)
  }
}

call_or_default <- function(FUN, X, ...) {
  if (is.null(FUN)) X else match.fun(FUN)(X, ...)
}

eval_formula <- function(x, var, data, envir = environment(x)) {
  if (length(x) == 2L) {
    eval(x[[2L]], if (!missing(data) && is.list(data)) data else NULL, envir)
  } else if (is.symbol(symbol <- x[[2L]])) {
    eval_args <- list(var)
    names(eval_args) <- as.character(symbol)
    eval(x[[3L]], eval_args, envir)
  } else {
    stop("The formula should be either '~ expr' or 'x ~ expr'", call. = FALSE)
  }
}

get_false_entries <- function(x) {
  y <- vapply(x, is_false, logical(1L))
  y <- names(x)[y]
  y[nzchar(y)]
}

is_false <- function(x) {
  is.logical(x) && length(x) == 1L && x == FALSE
}

get_digits <- function(x) {
  ifelse(grepl(".", x, fixed = TRUE),
    nchar(gsub("^.*\\.([0-9]*).*$", "\\1", x)), 0L)
}

seq_list <- function(x = character()) {
  lst <- as.list(seq_along(x))
  names(lst) <- x
  lst
}

copy_dim <- function(src, target, use.names = TRUE) {
  if (is.array(src)) {
    dim(target) <- dim(src)
    if (use.names) dimnames(target) <- dimnames(src)
  } else {
    if (use.names) names(target) <- names(src)
  }
  target
}

context("utils")

test_that("call_or_default", {
  expect_identical(call_or_default(NULL, 0), 0)
  expect_identical(call_or_default(sin, 1), sin(1))
})

test_that("get_digtis", {
  expect_equal(get_digits(c("0.1", "0.01", ".501", ".5001", "123.321", "123", "012", "5.")),
    c(1, 2, 3, 4, 3, 0, 0, 0))
})

test_that("copy_dim", {
  x <- c(1, 2, 3)
  y <- format(x)
  expect_true(!is.array(copy_dim(x, y)))

  x <- c(1, 2, 3, 4)
  m <- matrix(x, 2, dimnames = list(c("a", "b")))
  y <- format(x)
  y2 <- copy_dim(m, y)
  expect_true(is.matrix(y2))
  expect_identical(dim(y2), dim(m))
  expect_identical(dimnames(y2), dimnames(m))

  x <- rnorm(24)
  arr <- array(x, c(2, 3, 4),
    dimnames = list(letters[1:2], letters[1:3], letters[1:4]))
  y <- format(x)
  y2 <- copy_dim(arr, y)
  expect_true(is.array(y2))
  expect_identical(dim(y2), dim(arr))
  expect_identical(dimnames(y2), dimnames(arr))
})

#' Create a string-representation of CSS style
#'
#' Most HTML elements can be stylized by a set of CSS style
#' properties. This function helps build CSS strings using
#' conventional argument-passing in R.
#'
#' @details
#' The general usage of CSS styling is
#'
#' \code{<span style = "color: red; border: 1px">Text</span>}
#'
#' The text color can be specified by `color`, the border of
#' element by `border`, and etc.
#'
#' Basic styles like \code{color}, \code{border}, \code{background}
#' work properly and mostly consistently in modern web browsers.
#' However, some style properties may not work consistently in
#' different browsers.
#' @param ... style attributes in form of \code{name = value}. Many CSS properties
#' contains \code{'-'} in the middle of their names. In this case, use
#' \code{"the-name" = value} instead. \code{NA} will cancel the attribute.
#' @return a string-representation of css styles
#' @examples
#' style(color = "red")
#' style(color = "red", "font-weight" = "bold")
#' style("background-color" = "gray", "border-radius" = "4px")
#' style("padding-right" = "2px")
#'
#' formattable(mtcars, list(
#'   mpg = formatter("span",
#'     style = x ~ style(color = ifelse(x > median(x), "red", NA)))))
#' @seealso \href{https://www.w3.org/Style/CSS/all-properties}{List of CSS properties},
#'   \href{https://www.w3schools.com/cssref/}{CSS Reference}
#' @export
style <- function(...) {
  dots <- list(...)
  dots <- dots[vapply(dots, length, integer(1L)) > 0L]
  as.character(.mapply(function(...) {
    args <- list(...)
    args <- args[!is.na(args)]
    argnames <- names(args)
    argnames <- if (is.null(argnames)) "" else gsub(".", "-", argnames, fixed = TRUE)
    attrs <- .mapply(function(name, value) {
      paste0(name, ifelse(nzchar(name), ": ", ""), paste0(value, collapse = " "))
    }, list(name = argnames, value = args), NULL)
    paste0(attrs, collapse = "; ")
  }, dots, NULL))
}

#' Create icon-text elements
#' @param icon a character vector or list of character vectors
#' of icon names.
#' @param text a character vector of contents.
#' @param simplify logical to indicating whether to return
#' the only element if a single-valued list is resulted.
#' @param ... additional parameters (reserved)
#' @param provider the provider of icon set.
#' @param class_template a character value to specifiy to template of the class
#' with \code{"{provider}"} to represent \code{provider} value and \code{"{icon}"} to
#' represent \code{icon} values.
#' @seealso \href{https://getbootstrap.com/docs/3.4/components/#glyphicons}{Glyphicons in Bootstrap},
#' \href{https://glyphicons.com/}{Glyphicons}
#' @importFrom htmltools tagList tag
#' @export
#' @examples
#' icontext("plus")
#' icontext(c("star","star-empty"))
#' icontext(ifelse(mtcars$mpg > mean(mtcars$mpg), "plus", "minus"), mtcars$mpg)
#' icontext(list(rep("star",3), rep("star",2)), c("item 1", "item 2"))
icontext <- function(icon, text = list(NULL), ..., simplify = TRUE,
  provider = getOption("formattable.icon.provider", "glyphicon"),
  class_template = getOption("formattable.icon.class_template", "{provider} {provider}-{icon}")) {
  class_template <- gsub("{provider}", provider, class_template, fixed = TRUE)
  x <- .mapply(function(icon, text) {
    tagList(
      lapply(icon, function(ico)
        tag("i",
          list(class = gsub("{icon}", ico, class_template, fixed = TRUE)))), text)
  }, list(icon, text), NULL)
  if (simplify && length(x) == 1L) x[[1L]] else x
}

check_rgb <- function(x, alpha = TRUE) {
  grepl(if (alpha) "^#([0-9a-fA-F]{2}){3,4}$" else "^#([0-9a-fA-F]{2}){3}$", x)
}

check_rgba <- function(x) {
  grepl("^#([0-9a-fA-F]{2}){4}$", x)
}

#' @importFrom grDevices col2rgb
str2rgb <- function(x, alpha = NULL) {
  is_rgb <- check_rgb(x)
  if (missing(alpha) || is.null(alpha)) alpha <- any(check_rgba(x))
  rownames <- c("red", "green", "blue", if (alpha) "alpha", NULL)
  rows <- length(rownames)
  res <- matrix(0L, rows, length(x), byrow = TRUE,
    dimnames = list(rownames, names(x)))
  rgbs <- x[is_rgb]
  res[, is_rgb] <- matrix(strtoi(c(
    substr(rgbs, 2L, 3L),
    substr(rgbs, 4L, 5L),
    substr(rgbs, 6L, 7L),
    if (alpha) ifelse(nzchar(alphav <- substr(rgbs, 8L, 9L)), alphav, "FF") else NULL), 16L),
    nrow = rows, byrow = TRUE)
  res[, !is_rgb] <- col2rgb(x[!is_rgb], alpha = alpha)
  res
}

#' Create a matrix from vector to represent colors in gradient
#' @param x a numeric vector.
#' @param min.color color of minimum value.
#' @param max.color color of maximum value.
#' @param alpha logical of whether to include alpha channel. \code{NULL}
#' to let the function decide by input.
#' @param use.names logical of whether to preserve names of input vector.
#' @param na.rm logical indicating whether to ignore missing values as \code{x}
#' is normalized. (defult is \code{TRUE})
#' @return a matrix with rgba columns in which each row corresponds to the rgba
#' value (0-255) of each element in input vector \code{x}. Use \code{csscolor}
#' to convert the matrix to css color strings compatible with web browsers.
#' @seealso \code{\link{csscolor}}
#' @export
#' @examples
#' gradient(c(1,2,3,4,5), "white", "red")
#' gradient(c(5,4,3,2,1), "white", "red")
#' gradient(c(1,3,2,4,5), "white", "red")
#' gradient(c(1,3,2,4,5), rgb(0,0,0,0.5), rgb(0,0,0,1), alpha = TRUE)
gradient <- function(x, min.color, max.color, alpha = NULL, use.names = TRUE, na.rm = TRUE) {
  if (!is.numeric(x)) stop("x should be numeric")
  x <- unclass(x)
  color_range <- str2rgb(c(min = min.color, max = max.color), alpha = alpha)
  res <- if (length(x) > 0L) (color_range[, "max", drop = FALSE] -
      color_range[, "min", drop = FALSE]) %*% normalize(x, na.rm = na.rm) +
    matrix(rep(color_range[, "min", drop = FALSE], length(x)), ncol = length(x))
  else str2rgb(x, alpha = alpha)
  storage.mode(res) <- "integer"
  if (use.names) colnames(res) <- names(x)
  res
}

#' Generate CSS-compatible color strings
#' @param x color input
#' @param format the output format of color strings
#' @param use.names logical of whether to preserve the names of input
#' @return a character vector of CSS-compatible color strings
#' @export
#' @examples
#' csscolor(rgb(0, 0.5, 0.5))
#' csscolor(c(rgb(0, 0.2, 0.2), rgb(0, 0.5, 0.2)))
#' csscolor(rgb(0, 0.5, 0.5, 0.2))
#' csscolor(gradient(c(1,2,3,4,5), "white", "red"))
csscolor <- function(x, format = c("auto", "hex", "rgb", "rgba"),
  use.names = TRUE)
  UseMethod("csscolor")

#' @export
csscolor.character <- function(x, format = c("auto", "hex", "rgb", "rgba"), use.names = TRUE) {
  format <- match.arg(format)
  alpha <- all(check_rgb(x)) && any(check_rgba(x))
  if (format == "auto") format <- if (alpha) "rgba" else "hex"
  switch(format, hex = x,
    csscolor.matrix(str2rgb(x, alpha = alpha), format = format, use.names = use.names))
}

#' @export
csscolor.matrix <- function(x, format = c("auto", "hex", "rgb", "rgba"),
  use.names = TRUE) {
  format <- match.arg(format)
  alpha <- "alpha" %in% rownames(x)
  if (format == "auto") format <- if (alpha) "rgba" else "hex"
  na_cols <- apply(x, 2L, function(col) any(is.na(col)))
  cols <- switch(format, hex = {
    hex <- format.hexmode(as.hexmode(x[c("red", "green", "blue"), , drop = FALSE]), width = 2L)
    sprintf("#%s%s%s",
      hex[1L, , drop = FALSE],
      hex[2L, , drop = FALSE],
      hex[3L, , drop = FALSE])
  }, rgb = sprintf("rgb(%d, %d, %d)",
    x["red", , drop = FALSE],
    x["green", ,drop = FALSE],
    x["blue", ,drop = FALSE]),
    rgba = sprintf("rgba(%d, %d, %d, %g)",
      x["red", ,drop = FALSE],
      x["green", ,drop = FALSE],
      x["blue", ,drop = FALSE],
      if (alpha) round(x["alpha", ,drop = FALSE] / 255L, 2L) else 1))
  cols[na_cols] <- NA
  if (use.names) names(cols) <- colnames(x)
  cols
}

context("style")

test_that("style", {
  expect_equal(style(color = "red"), "color: red")
  expect_equal(style(color = "red", border = "1px"), "color: red; border: 1px")
  expect_equal(style(color = NA, border = "1px"), "border: 1px")
  expect_equal(style(color = c("red", "green")), c("color: red", "color: green"))
  expect_equal(style(color = c("red", "green"), border = "1px"),
    c("color: red; border: 1px", "color: green; border: 1px"))
  expect_equal(style(color = c("red", "green"), border = c("1px", "2px")),
    c("color: red; border: 1px", "color: green; border: 2px"))
  expect_equal(style(color = c("red", NA), border = "1px"),
    c("color: red; border: 1px", "border: 1px"))
  expect_equal(style(color = c("red", NA), border = c("1px", "2px")),
    c("color: red; border: 1px", "border: 2px"))
  expect_equal(style(color = "red", "border: 0px"), "color: red; border: 0px")
  expect_equal(style("color: red", "border: 0px"), "color: red; border: 0px")
  expect_equal(style(), character())
  expect_equal(style(x = 1:3, y = integer()), style(x = 1:3))
})

test_that("icontext", {
  expect_equal(as.character(icontext("plus")),
    as.character(htmltools::tags$i(class = "glyphicon glyphicon-plus")))

  icon_names <- c("plus", "minus")
  texts <- c("hello", "world")

  expect_equal(vapply(icontext(icon_names), as.character, character(1L)),
    vapply(icon_names, function(icon) {
      as.character(htmltools::tags$i(class = paste0("glyphicon glyphicon-", icon)))
    }, character(1L), USE.NAMES = FALSE))


  expect_equal(vapply(icontext(icon_names, texts), as.character, character(1L)),
    vapply(.mapply(function(icon, text) {
      htmltools::tagList(htmltools::tags$i(class = paste0("glyphicon glyphicon-", icon)), text)
    }, list(icon_names, texts), NULL), as.character, character(1L)))

  stars <- c(2, 5, 3)
  icon_names <- lapply(stars, function(n) rep("plus", n))
  texts <- sprintf("level %d", stars)

  expect_equal(vapply(icontext(icon_names), as.character, character(1L)),
    vapply(icon_names, function(icon) {
      as.character(htmltools::tagList(lapply(icon, function(i)
        htmltools::tags$i(class = paste0("glyphicon glyphicon-", i)))))
    }, character(1L)))

  expect_equal(vapply(icontext(icon_names, texts), as.character, character(1L)),
    vapply(.mapply(function(icon, text) {
      htmltools::tagList(lapply(icon, function(i)
        htmltools::tags$i(class = paste0("glyphicon glyphicon-", i))), text)
    }, list(icon_names, texts), NULL), as.character, character(1L)))

  stars <- 3
  icon_names <- lapply(stars, function(n) rep("plus", n))
  texts <- sprintf("level %d", stars)

  expect_equal(vapply(icontext(icon_names, texts, simplify = FALSE), as.character, character(1L)),
    vapply(.mapply(function(icon, text) {
      htmltools::tagList(lapply(icon, function(i)
        htmltools::tags$i(class = paste0("glyphicon glyphicon-", i))), text)
    }, list(icon_names, texts), NULL), as.character, character(1L)))

})

test_that("gradient", {
  expect_equal(gradient(c(1, 2, 3), "white", "red"),
    matrix(c(255, 255, 255, 255, 127, 127, 255, 0, 0), 3, 3,
      dimnames = list(c("red", "green", "blue"))))
  expect_equal(gradient(c(3, 2, 1), "white", "red"),
    matrix(c(255, 0, 0, 255, 127, 127, 255, 255, 255), 3, 3,
      dimnames = list(c("red", "green", "blue"))))
  expect_equal(gradient(c(1, 3, 2), "white", "red"),
    matrix(c(255, 255, 255, 255, 0, 0, 255, 127, 127), 3, 3,
      dimnames = list(c("red", "green", "blue"))))
  expect_equal(gradient(c(1, 2, 3, 4), rgb(1, 1, 0, 0), rgb(1, 0, 0, 1), alpha = TRUE),
    matrix(c(255, 255, 0, 0, 255, 170, 0, 85, 255, 85, 0, 170, 255, 0, 0, 255), 4, 4,
      dimnames = list(c("red", "green", "blue", "alpha"))))
  expect_equal(gradient(c(x = 1, y = 2, z = 3), "white", "red", use.names = TRUE),
    matrix(c(255, 255, 255, 255, 127, 127, 255, 0, 0), 3, 3,
      dimnames = list(c("red", "green", "blue"), c("x", "y", "z"))))
  expect_equal(gradient(c(x = 1, y = 2, z = 3), "white", "red", use.names = FALSE),
    matrix(c(255, 255, 255, 255, 127, 127, 255, 0, 0), 3, 3,
      dimnames = list(c("red", "green", "blue"))))
  expect_equal(gradient(c(1:3, NA), "white", "red"),
    matrix(c(255, 255, 255, 255, 127, 127, 255, 0, 0, NA, NA, NA), 3, 4,
      dimnames = list(c("red", "green", "blue"))))
  expect_equal(ncol(gradient(numeric(), "red", "blue")), 0L)
  expect_error(gradient(c(1:3, NA), "white", "red", na.rm = FALSE), "missing value")
})

test_that("csscolor", {
  expect_equal(csscolor(rgb(0, 0.5, 0.5)), "#008080")
  expect_equal(csscolor(c(rgb(0, 0.5, 0.5), rgb(0, 0.2, 0.5))), c("#008080", "#003380"))
  expect_equal(csscolor(rgb(0, 0.5, 0.5, 1)), "rgba(0, 128, 128, 1)")
  expect_equal(csscolor(gradient(c(1, 2, 3, 4, 5), "white", "red")),
    c("#ffffff", "#ffbfbf", "#ff7f7f", "#ff3f3f", "#ff0000"))
  expect_equal(csscolor(gradient(c(1, 3, 2), rgb(0, 0.1, 0.2, 0.5), rgb(0, 0.1, 0.2, 1))),
    c("rgba(0, 26, 51, 0.5)", "rgba(0, 26, 51, 1)", "rgba(0, 26, 51, 0.75)"))
  expect_equal(csscolor(gradient(c(1:3, NA), "white", "red")),
    c("#ffffff", "#ff7f7f", "#ff0000", NA))
  expect_equal(csscolor(gradient(0.1, "black", "white")), "#ffffff")
})

#' The formattable package
#'
#' This package is designed for applying formatting on vectors and
#' data frames to make data presentation easier, richer, more flexible
#' and hopefully convey more information.
#'
#' @name formattable-package
#' @docType package
#' @details
#' Atomic vectors are basic units to store data. Some data can be read more easily
#' with formatting. A numeric vector, for example, stores a group of
#' percentage numbers yet still shows in the form of typical floating numbers. This
#' package provides functions to create data structures with predefined
#' formatting rules so that these objects stores the original data but are printed
#' with formatting.
#'
#' On the other hand, in a typical workflow of dynamic document production, \code{knitr}
#' and \code{rmarkdown} are powerful tools to render documents with R code to different
#' types of portable documents.
#'
#' \code{knitr} package is able to render a RMarkdown document (markdown document
#' with R code chunks to be executed sequentially) to Markdown document.
#' \code{rmarkdown} calls \code{pandoc} to render markdown document to HTML web page.
#' To put a table from a \code{data.frame} on the page, one may call \code{knitr::kable}
#' to produce its markdown representation. By default the resulted table is in a plain
#' theme with no additional formatting. However, in some cases, additional formatting
#' may help clarify the information and make contrast of the data.
NULL

#' Generic function to create an htmlwidget
#'
#' This function is a generic function to create an \code{htmlwidget}
#' to allow HTML/JS from R in multiple contexts.
#'
#' @param x an object.
#' @param ... arguments to be passed to \code{\link[DT]{datatable}}
#' @export
#' @return a \code{\link[DT]{datatable}} object
as.datatable <- function(x, ...) {
  UseMethod("as.datatable")
}

#' Convert \code{formattable} to a \code{\link[DT]{datatable}} htmlwidget
#'
#' @param x a \code{formattable} object to convert
#' @param escape \code{logical} to escape \code{HTML}. The default is
#'          \code{FALSE} since it is expected that \code{formatters} from
#'          \code{formattable} will produce \code{HTML} tags.
#' @param ... additional arguments passed to to \code{\link[DT]{datatable}}
#' @return a \code{\link[DT]{datatable}} object
#' @export
as.datatable.formattable <- function(x, escape = FALSE, ...) {
  stopifnot(is.formattable(x), requireNamespace("DT"))
  DT::datatable(
    render_html_matrix.formattable(x),
    escape = escape,
    ...
  )
}

#' Vectorized map from element to case by index or string value
#'
#' This function is a vectorized version of \code{switch}, that is, for
#' each element of input vector, \code{switch} is evaluated and the results are
#' combined.
#'
#' @param EXPR an expression evaluated to be character or numeric vector/list.
#' @param ... The list of alternatives for each \code{switch}.
#' @param SIMPLIFY \code{TRUE} to simplify the resulted list to vector, matrix
#' or array if possible.
#' @seealso \code{\link{switch}}
#' @export
#' @examples
#' x <- c("normal","normal","error","unknown","unknown")
#' vmap(x, normal = 0, error = -1, unknown = -2)
#'
#' x <- c(1,1,2,1,2,2,1,1,2)
#' vmap(x, "type-A", "type-B")
vmap <- function(EXPR, ..., SIMPLIFY = TRUE) {
  if (is.factor(EXPR)) EXPR <- as.character.factor(EXPR)
  res <- lapply(EXPR, switch, ...)
  if (SIMPLIFY) simplify2array(res) else res
}

#' Quantile ranks of a vector
#'
#' The quantile rank of a number in a vector is the relative
#' position of ranking resulted from rank divided by the length
#' of vector.
#' @param x a vector
#' @param ... additional parameters passed to \code{rank}
#' @seealso \code{\link{rank}}
#' @export
#' @examples
#' qrank(mtcars$mpg)
qrank <- function(x, ...) {
  rank(x = x, ...) / length(x)
}

#' Normalize a vector to fit zero-to-one scale
#'
#' @param x a numeric vector
#' @param min numeric value. The lower bound of the interval to normalize \code{x}.
#' @param max numeric value. The upper bound of the interval to normalize \code{x}.
#' @param na.rm a logical indicating whether missing values should be removed
#' @export
#' @examples
#' normalize(mtcars$mpg)
normalize <- function(x, min = 0, max = 1, na.rm = FALSE) {
  if (all(is.na(x))) return(rep(0, length(x)))
  if (!is.numeric(x)) stop("x must be numeric")
  x <- unclass(x)
  if (min > max) stop("min <= max must be satisfied")
  if (all(x == 0, na.rm = na.rm)) return(x)
  xmax <- max(x, na.rm = na.rm)
  xmin <- min(x, na.rm = na.rm)
  if (xmax == xmin) return(rep(1, length(x)))
  min + (max - min) * (x - xmin) / (xmax - xmin)
}

#' Rescale a vector relative to the maximal absolute value in the vector
#'
#' @param x a numeric vector
#' @param na.rm a logical indicating whether missing values should be removed
#' @export
#' @examples
#' proportion(mtcars$mpg)
proportion <- function(x, na.rm = FALSE) {
  x / max(abs(x), na.rm = na.rm)
}

context("transform")

test_that("vmap", {
  expect_equal(vmap(c(1, 1, 2, 2, 3), "a", "b", "c"), c("a", "a", "b", "b", "c"))
  expect_equal(vmap(c("a", "b", "a", "c"), a = 1, b = 2, c = 3), c(1, 2, 1, 3))
  expect_equal(vmap(c("a", "b", "a", "c", "d"), a = 1, b = 2, c = 3, 0), c(1, 2, 1, 3, 0))
  expect_equal(vmap(c("a", "b", "a", "c"), a = 1, b = 2, c = 3, SIMPLIFY = FALSE), list(1, 2, 1, 3))
})

test_that("qrank", {
  n <- 100
  x <- rnorm(n)
  expect_equal(qrank(x), rank(x) / length(x))
  expect_equal(qrank(-x), rank(-x) / length(x))

  x <- rnorm(n)
  x[sample(1:n, 10, replace = FALSE)] <- NA
  expect_equal(qrank(x), rank(x) / length(x))
  expect_equal(qrank(-x), rank(-x) / length(x))
  expect_equal(qrank(x, na.last = FALSE), rank(x, na.last = FALSE) / length(x))
  expect_equal(qrank(-x, na.last = FALSE), rank(-x, na.last = FALSE) / length(x))
})

test_that("normalize", {
  n <- 100
  x <- rnorm(n)
  expect_equal(normalize(x), (x - min(x)) / (max(x) - min(x)))
  expect_equal(normalize(x, 0.5, 0.8), 0.5 + 0.3 * normalize(x))
  expect_error(normalize(x, 0.5, 0.1))

  x <- rnorm(n)
  x <- x - min(x, na.rm = TRUE) + 1
  x[sample(1:n, 10, replace = FALSE)] <- NA
  expect_error(normalize(x), "missing value")
  expect_equal(normalize(x, na.rm = TRUE),
    (x - min(x, na.rm = TRUE)) / (max(x, na.rm = TRUE) - min(x, na.rm = TRUE)))

  expect_equal(normalize(c(NA, NA, NA), na.rm = TRUE), c(0, 0, 0))
})

test_that("proportion", {
  n <- 100
  x <- rbinom(n, 100, 0.8)
  expect_equal(proportion(x), x / max(abs(x)))

  x <- rnorm(n)
  expect_equal(proportion(x), x / max(abs(x)))
})

#' Generic function to create formattable object
#'
#' This function is a generic function to create \code{formattable}
#' object, i.e. an object to which a formatting function and
#' related attribute are attached. The object works as ordinary object
#' yet has specially defined behavior as being printed or converted to
#' a string representation.
#'
#' @param x an object.
#' @param ... arguments to be passed to methods.
#' @export
#' @return a \code{formattable} object
formattable <- function(x, ...)
  UseMethod("formattable")

#' Test for objects of 'formattable' class
#' @param x an object
#' @return \code{TRUE} if \code{x} has class 'formattable';
#' \code{FALSE} otherwise.
#' @export
#' @examples
#' is.formattable(10)
#' is.formattable(formattable(10))
is.formattable <- function(x) {
  inherits(x, "formattable")
}

#' Create a formattable object
#' @inheritParams formattable
#' @param ... arguments to be passed to \code{formatter}.
#' @param formatter formatting function, \code{formatC} in default.
#' @param preproc pre-processor function that prepares \code{x} for
#' formatting function.
#' @param postproc post-processor function that transforms formatted
#' output for printing.
#' @export
#' @return a \code{formattable} object that inherits from the original
#' object.
#' @examples
#' formattable(rnorm(10), formatter = "formatC", digits = 1)
formattable.default <- function(x, ..., formatter,
  preproc = NULL, postproc = NULL) {
  create_obj(x, "formattable",
    list(formatter = formatter,
      format = list(...), preproc = preproc, postproc = postproc))
}

#' Create a formattable numeric vector
#' @inheritParams formattable.default
#' @param x a numeric vector.
#' @param formatter formatting function, \link{formatC} in default.
#' @export
#' @return a \code{formattable} numeric vector.
#' @examples
#' formattable(rnorm(10), format = "f", digits = 1)
#' formattable(rnorm(10), format = "f",
#'   flag="+", digits = 1)
#' formattable(1:10,
#'   postproc = function(str, x) paste0(str, "px"))
#' formattable(1:10,
#'   postproc = function(str, x)
#'     paste(str, ifelse(x <= 1, "unit", "units")))
formattable.numeric <- function(x, ..., formatter = "formatC",
  preproc = NULL, postproc = NULL) {
  create_obj(x, "formattable",
    list(formatter = formatter,
      format = list(...), preproc = preproc, postproc = postproc))
}

#' @export
formattable.table <- function(x, ..., formatter = "format",
  preproc = NULL, postproc = NULL) {
  create_obj(x, "formattable",
    list(formatter = formatter,
      format = list(...), preproc = preproc, postproc = postproc))
}

#' Create a formattable logical vector
#' @inheritParams formattable.default
#' @param x a logical vector.
#' @param formatter formatting function, \code{formattable::ifelse} in default.
#' @export
#' @return a \code{formattable} logical vector.
#' @examples
#' logi <- c(TRUE, TRUE, FALSE)
#' flogi <- formattable(logi, "yes", "no")
#' flogi
#' !flogi
#' any(flogi)
#' all(flogi)
formattable.logical <- function(x, ..., formatter = "ifelse",
  preproc = NULL, postproc = NULL) {
  create_obj(x, "formattable",
    list(formatter = formatter,
      format = list(...), preproc = preproc, postproc = postproc))
}

#' Create a formattable factor object
#' @inheritParams formattable.default
#' @param x a factor object.
#' @param formatter formatting function, \code{vmap} in default.
#' @export
#' @return a \code{formattable} factor object.
#' @examples
#' formattable(as.factor(c("a", "b", "b", "c")),
#'   a = "good", b = "fair", c = "bad")
formattable.factor <- function(x, ..., formatter = "vmap",
  preproc = NULL, postproc = NULL) {
  create_obj(x, "formattable",
    list(formatter = formatter,
      format = list(...), preproc = preproc, postproc = postproc))
}

#' Create a formattable Date vector
#' @inheritParams formattable.default
#' @param x a vector of class \code{Date}.
#' @param formatter formatting function, \code{format.Date} in default.
#' @export
#' @return a \code{formattable} Date vector
#' @examples
#' dates <- as.Date("2015-04-10") + 1:5
#' fdates <- formattable(dates, format = "%m/%d/%Y")
#' fdates
#' fdates + 30
formattable.Date <- function(x, ..., formatter = "format.Date",
  preproc = NULL, postproc = NULL) {
  create_obj(x, "formattable",
    list(formatter = formatter,
      format = list(...), preproc = preproc, postproc = postproc))
}

#' Create a formattable POSIXct vector
#' @inheritParams formattable.default
#' @param x a vector of class \code{POSIXct}.
#' @param formatter formatting function, \code{format.POSIXct} in default.
#' @export
#' @return a \code{formattable} POSIXct vector
#' @examples
#' times <- as.POSIXct("2015-04-10 09:30:15") + 1:5
#' ftimes <- formattable(times, format = "%Y%m%dT%H%M%S")
#' ftimes
#' ftimes + 30
formattable.POSIXct <- function(x, ..., formatter = "format.POSIXct",
  preproc = NULL, postproc = NULL) {
  create_obj(x, "formattable",
    list(formatter = formatter,
      format = list(...), preproc = preproc, postproc = postproc))
}

#' Create a formattable POSIXlt vector
#' @inheritParams formattable.default
#' @param x a vector of class \code{POSIXlt}.
#' @param formatter formatting function, \code{format.POSIXlt} in default.
#' @export
#' @return a \code{formattable} POSIXlt vector
#' @examples
#' times <- as.POSIXlt("2015-04-10 09:30:15") + 1:5
#' ftimes <- formattable(times, format = "%Y%m%dT%H%M%S")
#' ftimes
#' ftimes + 30
formattable.POSIXlt <- function(x, ..., formatter = "format.POSIXlt",
  preproc = NULL, postproc = NULL) {
  create_obj(x, "formattable",
    list(formatter = formatter,
      format = list(...), preproc = preproc, postproc = postproc))
}

#' @export
formattable.formattable <- function(x, ..., formatter, preproc, postproc) {
  attrs <- attr(x, "formattable", exact = TRUE)
  if (is.null(attrs)) return(NextMethod("formattable"))
  if (!missing(formatter)) attrs$formatter <- formatter
  if (!missing(...)) attrs$format <- list(...)
  if (!missing(preproc))
    attrs$preproc <- if (is.list(preproc)) c(preproc, attrs$preproc) else preproc
  if (!missing(postproc))
    attrs$postproc <- if (is.list(postproc)) c(attrs$postproc, postproc) else postproc
  attr(x, "formattable") <- attrs
  x
}

#' @export
as.character.formattable <- function(x, ...) {
  format.formattable(x)
}

print_formattable <- function(x, ...)
  UseMethod("print_formattable")

print_formattable.default <- function(x, ...) {
  if (length(x) == 0L) cat(sprintf("%s(0)", paste0(class(x), collapse = " ")))
  else {
    args <- list(...)
    print_args <- attr(x, "formattable", exact = TRUE)$print
    if (is.null(print_args)) print_args <- list()
    print_args[names(args)] <- args
    if (is.null(print_args$quote)) print_args$quote <- is.character(x)
    do.call(print, c(list(format.formattable(x)), print_args))
  }
  invisible(x)
}

print_formattable.data.frame <- function(x, ...) {
  print(as.htmlwidget(x), ...)
}

#' @export
print.formattable <- function(x, ...) {
  print_formattable(x, ...)
}

knit_print_formattable <- function(x, ...)
  UseMethod("knit_print_formattable")

knit_print_formattable.default <- print_formattable.default

#' @importFrom knitr asis_output
knit_print_formattable.data.frame <- function(x, ...) {
  format <- attr(x, "formattable", exact = TRUE)$format
  caption <- if (isTRUE(format$format == "pandoc" && nzchar(format$caption))) "<!-- -->\n\n" else ""
  asis_output(sprintf("\n%s%s\n", caption, paste0(as.character(x), collapse = "\n")))
}

#' @export
#' @importFrom knitr knit_print
knit_print.formattable <- function(x, ...)
  knit_print_formattable(x, ...)

#' @export
format.formattable <- function(x, ...,
  format = NULL,
  justify = "none", na.encode = FALSE, trim = FALSE, use.names = TRUE) {
  attrs <- attr(x, "formattable", exact = TRUE)
  if (length(x) == 0L || is.null(attrs) || is.null(attrs$formatter))
    return(NextMethod("format"))
  format_args <- attrs$format
  format_args[names(format)] <- format
  value <- remove_class(x, "formattable")
  if (length(attrs$preproc)) {
    preproc_list <- if (is.list(attrs$preproc)) attrs$preproc else list(attrs$preproc)
    for (preproc in preproc_list) value <- call_or_default(preproc, value)
  }
  str <- do.call(attrs$formatter, c(list(value), format_args))
  if (x_atomic <- is.atomic(x)) str <- remove_attribute(str, "formattable")
  if (length(attrs$postproc)) {
    postproc_list <- if (is.list(attrs$postproc)) attrs$postproc else list(attrs$postproc)
    for (postproc in postproc_list) str <- call_or_default(postproc, str, value)
  }
  str <- as.character(str)
  if (x_atomic) {
    str <- copy_dim(x, str, use.names)
  }
  str
}

#' @export
as.list.formattable <- function(x, ...) {
  lapply(seq_along(x), function(i) x[[i]])
}

#' @export
`[.formattable` <- function(x, ...) {
  value <- NextMethod("[")
  if (is.atomic(x) || is.data.frame(x))
    copy_obj(x, value, "formattable")
  else value
}

#' @export
`[<-.formattable` <- function(x, ..., value) {
  value <- NextMethod("[<-")
  reset_class(x, value, "formattable")
}

#' @export
`[[.formattable` <- function(x, ...) {
  value <- NextMethod("[[")
  if (is.atomic(x))
    copy_obj(x, value, "formattable")
  else value
}

#' @export
`[[<-.formattable` <- function(x, ..., value) {
  value <- NextMethod("[[<-")
  reset_class(x, value, "formattable")
}

#' @export
c.formattable <- function(..., recursive = FALSE) {
  fcreate_obj(c, "formattable", ...)
}

#' @export
`+.formattable` <- function(x, y) {
  cop_create_obj(`+`, "formattable", x, y)
}

#' @export
`-.formattable` <- function(x, y) {
  cop_create_obj(`-`, "formattable", x, y)
}

#' @export
`*.formattable` <- function(x, y) {
  cop_create_obj(`*`, "formattable", x, y)
}

#' @export
`/.formattable` <- function(x, y) {
  fcreate_obj(`/`, "formattable", x, unclass(y))
}

#' @export
`%%.formattable` <- function(x, y) {
  fcreate_obj(`%%`, "formattable", x, unclass(y))
}

#' @export
rep.formattable <- function(x, ...) {
  fcreate_obj(rep, "formattable", x, ...)
}

#' @export
`&.formattable` <- function(x, y) {
  fcreate_obj(`&`, "formattable", x, unclass(y))
}

#' @export
`|.formattable` <- function(x, y) {
  fcreate_obj(`|`, "formattable", x, unclass(y))
}

#' @export
all.formattable <- function(...) {
  fcreate_obj(all, "formattable", ...)
}

#' @export
any.formattable <- function(...) {
  fcreate_obj(any, "formattable", ...)
}

#' @export
max.formattable <- function(...) {
  fcreate_obj(max, "formattable", ...)
}

#' @export
min.formattable <- function(...) {
  fcreate_obj(min, "formattable", ...)
}

#' @export
sum.formattable <- function(...) {
  fcreate_obj(sum, "formattable", ...)
}

#' @export
mean.formattable <- function(x, ...) {
  fcreate_obj(mean, "formattable", x, ...)
}

#' @export
unique.formattable <- function(x, ...) {
  fcreate_obj(unique, "formattable", x, ...)
}

#' @export
diff.formattable <- function(x, ...) {
  fcreate_obj(diff, "formattable", x, ...)
}

#' @export
cummax.formattable <- function(x, ...) {
  fcreate_obj(cummax, "formattable", x, ...)
}

#' @export
cummin.formattable <- function(x, ...) {
  fcreate_obj(cummin, "formattable", x, ...)
}

#' @export
cumsum.formattable <- function(x, ...) {
  fcreate_obj(cumsum, "formattable", x, ...)
}

#' @importFrom stats median
#' @export
median.formattable <- function(x, ...) {
  fcreate_obj(median, "formattable", x, ...)
}

#' @importFrom stats quantile
#' @export
quantile.formattable <- function(x, ...) {
  fcreate_obj(quantile, "formattable", x, ...)
}

render_html_matrix <- function(x, ...)
  UseMethod("render_html_matrix")

render_html_matrix.data.frame <- function(x, formatters = list(), digits = getOption("digits"), ...) {
  stopifnot(is.data.frame(x))
  if (nrow(x) == 0L) formatters <- list()
  cols <- colnames(x)
  mat <- vapply(x, format, character(nrow(x)), digits = digits, trim = TRUE)
  dim(mat) <- dim(x)
  dimnames(mat) <- dimnames(x)
  for (fi in seq_along(formatters)) {
    fn <- names(formatters)[[fi]]
    f <- formatters[[fi]]
    if (is_false(f)) next
    else if (!is.null(fn) && nzchar(fn)) {
      if (fn %in% cols) {
        value <- x[[fn]]
        fv <- if (inherits(f, "formatter")) f(value, x, mat[, fn])
        else  if (inherits(f, "formula")) eval_formula(f, value, x)
        else match.fun(f)(value)
        mat[, fn] <- format(fv)
      }
    } else if (inherits(f, "formula")) {
      fenv <- environment(f)
      value <- as.matrix(if (length(f) == 2L) {
        row <- col <- TRUE
        f <- eval(f[[2L]], fenv)
        x
      } else {
        if (is.call(f[[2L]])) {
          farea <- eval(f[[2L]], fenv)
          if (inherits(farea, "area")) {
            row <- eval(farea$row, seq_list(rownames(x)), farea$envir)
            col <- eval(farea$col, seq_list(colnames(x)), farea$envir)
            f <- eval(f[[3L]], fenv)
            x[row, col]
          } else {
            stop("Invalid area formatter specification. Use area(row, col) ~ formatter instead.", call. = FALSE)
          }
        } else {
          stop("Invalid formatter specification. Use area(row, col) ~ formatter instead.", call. = FALSE)
        }
      })
      fv <-  if (inherits(f, "formatter"))
        f(value, x, mat[row, col]) else match.fun(f)(value)
      mat[row, col] <- format(fv)
    }
  }
  nulls <- get_false_entries(formatters)
  mat[, setdiff(cols, nulls), drop = FALSE]
}

render_html_matrix.formattable <- function(x, ...) {
  stopifnot(is.formattable(x), is.data.frame(x))
  do.call(render_html_matrix.data.frame,
    c(list(x = remove_class(x, "formattable")), attr(x, "formattable")$format))
}

#' Format a data frame with formatter functions
#'
#' This is an table generator that specializes in creating
#' formatted table presented in HTML by default.
#' To generate a formatted table, columns or areas of the
#' input data frame can be transformed by formatter functions.
#' @importFrom knitr kable
#' @param x a \code{data.frame}.
#' @param formatters a list of formatter functions or formulas.
#' The existing columns of \code{x} will be applied the formatter
#' function in \code{formatters} if it exists.
#'
#' If a formatter is specified by formula, then the formula will be
#' interpreted as a lambda expression with its left-hand side being
#' a symbol and right-hand side being the expression using the symbol
#' to represent the column values. The formula expression will be evaluated
#' in the environment of the formula.
#'
#' If a formatter is \code{FALSE}, then the corresponding column will be hidden.
#'
#' Area formatter is specified in the form of
#' \code{area(row, col) ~ formatter} without specifying the column name.
#' @param format The output format: html, markdown or pandoc?
#' @param align The alignment of columns: a character vector consisting
#' of \code{'l'} (left), \code{'c'} (center), and/or \code{'r'} (right).
#' By default, all columns are right-aligned.
#' @param ... additional parameters to be passed to \code{knitr::kable}.
#' @param digits The number of significant digits to be used for numeric
#'     and complex values.
#' @param table.attr The HTML class of \code{<table>} created when
#'     \code{format = "html"}
#' @return a \code{knitr_kable} object whose \code{print} method generates a
#' string-representation of \code{data} formatted by \code{formatter} in
#' specific \code{format}.
#' @export
#' @examples
#' # mtcars (mpg in red)
#' format_table(mtcars,
#'    list(mpg = formatter("span", style = "color:red")))
#'
#' # mtcars (mpg in red if greater than median)
#' format_table(mtcars, list(mpg = formatter("span",
#'    style = function(x) ifelse(x > median(x), "color:red", NA))))
#'
#' # mtcars (mpg in red if greater than median, using formula)
#' format_table(mtcars, list(mpg = formatter("span",
#'    style = x ~ ifelse(x > median(x), "color:red", NA))))
#'
#' # mtcars (mpg in gradient: the higher, the redder)
#' format_table(mtcars, list(mpg = formatter("span",
#'    style = x ~ style(color = rgb(x/max(x), 0, 0)))))
#'
#' # mtcars (mpg background in gradient: the higher, the redder)
#' format_table(mtcars, list(mpg = formatter("span",
#'    style = x ~ style(display = "block",
#'    "border-radius" = "4px",
#'    "padding-right" = "4px",
#'    color = "white",
#'    "background-color" = rgb(x/max(x), 0, 0)))))
#'
#' # mtcars (mpg in red if vs == 1 and am == 1)
#' format_table(mtcars, list(mpg = formatter("span",
#'     style = ~ style(color = ifelse(vs == 1 & am == 1, "red", NA)))))
#'
#' # hide columns
#' format_table(mtcars, list(mpg = FALSE, cyl = FALSE))
#'
#' # area formatting
#' format_table(mtcars, list(area(col = vs:carb) ~ formatter("span",
#'   style = x ~ style(color = ifelse(x > 0, "red", NA)))))
#'
#' df <- data.frame(a = rnorm(10), b = rnorm(10), c = rnorm(10))
#' format_table(df, list(area() ~ color_tile("transparent", "lightgray")))
#' format_table(df, list(area(1:5) ~ color_tile("transparent", "lightgray")))
#' format_table(df, list(area(1:5) ~ color_tile("transparent", "lightgray"),
#'   area(6:10) ~ color_tile("transparent", "lightpink")))
#' @seealso \link{formattable}, \link{area}
format_table <- function(x, formatters = list(),
  format = c("html", "markdown", "pandoc"), align = "r", ...,
  digits = getOption("digits"), table.attr = 'class="table table-condensed"') {
  format <- match.arg(format)
  mat <- render_html_matrix(x, formatters, digits)
  kable(mat, format = format, align = align, escape = FALSE, ...,
    table.attr = table.attr)
}

#' Create a formattable data frame
#'
#' This function creates a formattable data frame by attaching
#' column or area formatters to the data frame. Each time the data frame
#' is printed or converted to string representation, the formatter
#' function will use the formatter functions to generate
#' formatted cells.
#'
#' @details
#' The formattable data frame is a data frame with lazy-bindings
#' of prespecified column formatters or area formatters.
#' The formatters will not be applied until the data frame is
#' printed to console or in a dynamic document. If the formatter
#' function has no side effect, the formattable data frame will
#' not be changed even if the formatters are applied to produce
#' the printed version.
#'
#' @inheritParams formattable.default
#' @param x a \code{data.frame}
#' @param formatter formatting function, \code{format_table} in default.
#' @export
#' @return a \code{formattable data.frame}
#' @examples
#' # mtcars (mpg in red)
#' formattable(mtcars,
#'    list(mpg = formatter("span", style = "color:red")))
#'
#' # mtcars (mpg in red if greater than median)
#' formattable(mtcars, list(mpg = formatter("span",
#'    style = function(x) ifelse(x > median(x), "color:red", NA))))
#'
#' # mtcars (mpg in red if greater than median, using formula)
#' formattable(mtcars, list(mpg = formatter("span",
#'    style = x ~ ifelse(x > median(x), "color:red", NA))))
#'
#' # mtcars (mpg in gradient: the higher, the redder)
#' formattable(mtcars, list(mpg = formatter("span",
#'    style = x ~ style(color = rgb(x/max(x), 0, 0)))))
#'
#' # mtcars (mpg background in gradient: the higher, the redder)
#' formattable(mtcars, list(mpg = formatter("span",
#'    style = x ~ style(display = "block",
#'    "border-radius" = "4px",
#'    "padding-right" = "4px",
#'    color = "white",
#'    "background-color" = rgb(x/max(x), 0, 0)))))
#'
#' # mtcars (mpg in red if vs == 1 and am == 1)
#' formattable(mtcars, list(mpg = formatter("span",
#'     style = ~ style(color = ifelse(vs == 1 & am == 1, "red", NA)))))
#'
#' # hide columns
#' formattable(mtcars, list(mpg = FALSE, cyl = FALSE))
#'
#' # area formatting
#' formattable(mtcars, list(area(col = vs:carb) ~ formatter("span",
#'   style = x ~ style(color = ifelse(x > 0, "red", NA)))))
#'
#' df <- data.frame(a = rnorm(10), b = rnorm(10), c = rnorm(10))
#' formattable(df, list(area() ~ color_tile("transparent", "lightgray")))
#' formattable(df, list(area(1:5) ~ color_tile("transparent", "lightgray")))
#' formattable(df, list(area(1:5) ~ color_tile("transparent", "lightgray"),
#'   area(6:10) ~ color_tile("transparent", "lightpink")))
#' @seealso \link{format_table}, \link{area}
formattable.data.frame <- function(x, ..., formatter = "format_table",
  preproc = NULL, postproc = NULL) {
  create_obj(x, "formattable",
    list(formatter = formatter, format = list(...),
      preproc = preproc, postproc = postproc))
}

context("formattable")

test_that("formattable.default", {
  obj <- structure(list(x = 1, y = 2), class = c("test_object", "list"))
  fobj <- formattable(obj, formatter = function(obj) {
    digits <- attr(obj, "formattable", TRUE)$format$digits
    if (is.null(digits)) digits <- 2L
    sprintf("text_object { x: %s, y: %s }", round(obj$x, digits), round(obj$y, digits))
  })
  expect_is(fobj, c("formattable", "test_object", "list"))
  expect_equal(fobj$x, obj$x)
  expect_equal(fobj$y, obj$y)
  expect_equal(format(fobj), "text_object { x: 1, y: 2 }")

  x <- c(1, 2, 3)
  obj <- structure(x, class = "formattable")
  expect_identical(format(obj), format(x))
  expect_identical(format(formattable(x)), format(formattable.numeric(obj)))
})

test_that("formattable.numeric", {
  obj <- formattable(numeric(), format = "f", digits = 2L)
  expect_is(obj, c("formattable", "numeric"))
  expect_identical(format(obj), character())

  num <- rnorm(10)
  obj <- formattable(num, format = "f", digits = 2L)
  expect_is(obj, c("formattable", "numeric"))
  expect_equal(format(obj), formatC(num, format = "f", digits = 2L))
  expect_equal(format(c(obj, 0.1)), formatC(c(num, 0.1), format = "f", digits = 2L))

  num <- 1:10
  obj <- formattable(num)
  obj_attr <- attr(obj, "formattable", TRUE)
  expect_is(obj, c("formattable", "integer"))
  obj[5] <- 0L
  expect_is(obj, c("formattable", "integer"))
  expect_identical(attr(obj, "formattable", TRUE), obj_attr)

  obj[5] <- 2.5
  expect_is(obj, c("formattable", "numeric"))
  expect_identical(attr(obj, "formattable", TRUE), obj_attr)

  obj[6] <- "abc"
  expect_is(obj, c("formattable", "character"))
  expect_identical(attr(obj, "formattable", TRUE), obj_attr)

  num <- 1:10
  obj <- formattable(num)
  obj_attr <- attr(obj, "formattable", TRUE)
  expect_is(obj, c("formattable", "integer"))
  obj[[5]] <- 0L
  expect_is(obj, c("formattable", "integer"))
  expect_identical(attr(obj, "formattable", TRUE), obj_attr)

  obj[[5]] <- 2.5
  expect_is(obj, c("formattable", "numeric"))
  expect_identical(attr(obj, "formattable", TRUE), obj_attr)

  obj[[6]] <- "abc"
  expect_is(obj, c("formattable", "character"))
  expect_identical(attr(obj, "formattable", TRUE), obj_attr)

  num <- rnorm(10)
  names(num) <- letters[seq_along(num)]
  obj <- formattable(num, format = "f", digits = 4L)
  expect_is(obj, c("formattable", "numeric"))
  expect_is(as.character(obj), "character")
  expect_equivalent(format(obj[1:4]), formatC(num[1:4], format = "f", digits = 4L))
  expect_equivalent(format(obj[[3]]), formatC(num[[3]], format = "f", digits = 4L))
  expect_equivalent(format(obj + 0.123456), formatC(num + 0.123456, format = "f", digits = 4L))
  expect_equivalent(format(obj - 0.123456), formatC(num - 0.123456, format = "f", digits = 4L))
  expect_equivalent(format(-obj), formatC(-num, format = "f", digits = 4L))
  expect_equivalent(format(obj * 0.123456), formatC(num * 0.123456, format = "f", digits = 4L))
  expect_equivalent(format(obj / 0.123456), formatC(num / 0.123456, format = "f", digits = 4L))
  expect_equivalent(formattable(1:10) %% 2, formattable(1:10 %% 2))
  expect_equivalent(rep(formattable(1), 3), formattable(rep(1, 3)))
  expect_equivalent(format(max(obj)), formatC(max(num), format = "f", digits = 4L))
  expect_equivalent(format(min(obj)), formatC(min(num), format = "f", digits = 4L))
  expect_equivalent(format(sum(obj)), formatC(sum(num), format = "f", digits = 4L))
  expect_equivalent(format(mean(obj)), formatC(mean(num), format = "f", digits = 4L))
  expect_equivalent(format(diff(obj)), formatC(diff(num), format = "f", digits = 4L))
  expect_equivalent(format(unique(obj)), formatC(unique(num), format = "f", digits = 4L))
  expect_equivalent(format(cummax(obj)), formatC(cummax(num), format = "f", digits = 4L))
  expect_equivalent(format(cummin(obj)), formatC(cummin(num), format = "f", digits = 4L))
  expect_equivalent(format(cumsum(obj)), formatC(cumsum(num), format = "f", digits = 4L))
  expect_equivalent(format(median(obj)), formatC(median(num), format = "f", digits = 4L))
  expect_equivalent(format(quantile(obj)), formatC(quantile(num), format = "f", digits = 4L))
  expect_equivalent(format(sort(obj)), formatC(sort(num), format = "f", digits = 4L))
  expect_equivalent(c(formattable(1), 2), formattable(c(1, 2)))
  expect_identical(cop_create_obj(`+`, "test", 1, 2), structure(3, class = c("test", "numeric")))
  expect_output(invisible(print(formattable(2, digits = 4, format = "f"))), "^\\[1\\] 2.0000$")
})

test_that("formattable.table", {
  # 1d table
  x <- rbinom(100, 8, 0.5)
  tx <- table(x)
  fx <- formattable(tx)
  expect_is(fx, c("formattable", "table"))
  expect_true(is.integer(fx))
  expect_identical(names(format(fx)), names(tx))
  expect_equal(format(fx), format(tx))

  # 2d table
  y <- rbinom(100, 3, 0.6)
  tx <- table(x, y)
  fx <- formattable(tx)
  expect_is(fx, c("formattable", "table"))
  expect_true(is.matrix(fx))
  expect_true(is.integer(fx))
  expect_identical(dimnames(fx), dimnames(tx))
  expect_equal(format(fx), format(tx))
})

test_that("formattable.logical", {
  logi <- rnorm(10) >= 0
  obj <- formattable(logi, "yes", "no")
  expect_is(obj, c("formattable", "logical"))
  expect_equal(format(obj), ifelse(logi, "yes", "no"))
  expect_equal(format(!obj), ifelse(!logi, "yes", "no"))
  expect_equal(format(all(obj)), ifelse(all(logi), "yes", "no"))
  expect_equal(format(any(obj)), ifelse(any(logi), "yes", "no"))
  expect_equivalent(format(obj & obj), ifelse(logi & logi, "yes", "no"))
  expect_equivalent(format(obj | obj), ifelse(logi | logi, "yes", "no"))
})

test_that("formattable.factor", {
  values <- as.factor(c("a", "b", "b", "c"))
  obj <- formattable(values, a = "good", b = "fair", c = "bad")
  expect_is(obj, c("formattable", "factor"))
  expect_equal(format(obj), vmap(values, a = "good", b = "fair", c = "bad"))
  expect_equal(format(c(obj, as.factor("c"))), vmap(c(values, as.factor("c")), a = "good", b = "fair", c = "bad"))
})

test_that("formattable.Date", {
  dt <- as.Date("2015-01-01") + 1:5
  obj <- formattable(dt, format = "%Y%m%d")
  expect_is(obj, c("formattable", "Date"))
  expect_equal(format(obj), format(dt, "%Y%m%d"))
})

test_that("formattable.POSIXct", {
  dt <- as.POSIXct("2015-01-01 09:15:20") + 1:5
  obj <- formattable(dt, format = "%Y%m%dT%H%M%S")
  expect_is(obj, c("formattable", "POSIXct"))
  expect_equal(format(obj), format(dt, "%Y%m%dT%H%M%S"))
})

test_that("formattable.POSIXlt", {
  dt <- as.POSIXlt("2015-01-01")
  obj <- formattable(dt, format = "%Y%m%dT%H%M%S")
  expect_is(obj, c("formattable", "POSIXlt"))
  expect_equal(format(obj), format(dt, "%Y%m%dT%H%M%S"))
})

test_that("foramttable operators", {
  expect_equal(format(formattable(1:10) + 0.5), formatC(1:10 + 0.5))
  expect_equal(format(0.5 + formattable(1:10)), formatC(0.5 + 1:10))
  expect_equal(format(percent(0.5) * 2), "100.00%")
  expect_equal(format(2 * percent(0.5)), "100.00%")
})

test_that("formattable methods", {
  expect_equal(as.list(percent(c(0.1, 0.2))),
    list(percent(0.1), percent(0.2)))
  expect_equal(lapply(percent(c(0.1, 0.2)), identity),
    list(percent(0.1), percent(0.2)))
})

test_that("render_html_matrix", {
  d <- data.frame(a = c(1, 2), b = c(1.23456, 2.5),
    c = c("a", "b"), d = as.Date("2016-01-05") + 0:1)
  fd <- render_html_matrix.data.frame(d)
  expect_true(is.matrix(fd) && is.character(fd))
  expect_equal(fd,
    matrix(c("1", "2",
      "1.23456", "2.50000",
      "a", "b",
      "2016-01-05", "2016-01-06"),
    nrow = 2L, ncol = 4L,
    dimnames = list(c("1", "2"), c("a", "b", "c", "d"))))
})

test_that("formattable.data.frame", {
  obj <- formattable(mtcars)
  expect_is(obj, c("formattable", "data.frame"))
  expect_is(format_table(mtcars), "knitr_kable")
  expect_is(format_table(obj), "knitr_kable")
  expect_is(formattable(mtcars, list(mpg = formatter("span",
    style = x ~ style(display = "block",
      "border-radius" = "4px",
      "padding-right" = "4px",
      color = "white",
      "background-color" = rgb(x / max(x), 0, 0)))))
  , "formattable")
  expect_is(formattable(mtcars, list(mpg = formatter("span",
    style = function(x) ifelse(x > median(x), "color:red", NA)))),
  "formattable")
  expect_is(format_table(mtcars, list(mpg = formatter("span",
    style = function(x) ifelse(x > median(x), "color:red", NA)))),
  "knitr_kable")
  expect_is(format_table(mtcars,
    list(vs = x ~ formattable(as.logical(x), "yes", "no"))),
  "knitr_kable")
  expect_is(format_table(mtcars, list(vs = ~"unknown")), "knitr_kable")
  expect_error(format_table(mtcars, list(vs = f(a, b) ~ "unknown")))

  df <- formattable(mtcars, list(mpg = color_tile("red", "green")))
  expect_identical(attr(df, "formattable", TRUE), attr(df[1:10, ], "formattable", TRUE))
  expect_identical(attr(df, "formattable", TRUE), attr(df[, c("cyl", "mpg")], "formattable", TRUE))
  expect_identical(attr(df, "formattable", TRUE), attr(df[1:10, c("cyl", "mpg")], "formattable", TRUE))
  knit_df <- knit_print.formattable(df)
  expect_is(knit_df, "knit_asis")
  expect_true(is.character(knit_df))

  df <- data.frame(id = integer(), name = character(), value = numeric())
  expect_is(format_table(formattable(df, list(value = color_tile("red", "blue")))), "knitr_kable")

  # formula
  df <- data.frame(a = rnorm(10, 0.1), b = rnorm(10, 0.1), c = rnorm(10, 0.1))
  format_table(df, list(~percent))

  # cross formatting
  format_table(df, list(b = formatter("span", style = ~ style(color = ifelse(a >= mean(a), "red", "green")))))

  # hiding columns
  format_table(df, list(a = FALSE))

  # area formatter
  df <- data.frame(a = rnorm(10, 0.1), b = rnorm(10, 0.1), c = rnorm(10, 0.1))
  expect_is(format_table(df, list(area(col = c("a", "b")) ~ percent)), "knitr_kable")
  expect_is(format_table(df, list(area(col = b:c) ~ percent)), "knitr_kable")
  expect_is(format_table(df, list(area(1:5) ~ percent)), "knitr_kable")
  expect_is(format_table(df, list(area(1:5, b:c) ~ percent)), "knitr_kable")

  expect_error(format_table(df, list(x ~ percent)))
  expect_error(format_table(df, list(get("df") ~ percent)))
})

test_that("formattable matrix", {
  m <- rnorm(100)
  dim(m) <- c(50, 2)
  colnames(m) <- c("a", "b")
  fm <- formattable(m)
  expect_is(fm, c("formattable", "matrix"))
  expect_true(is.matrix(fm))
  expect_equal(dim(fm), dim(m))
  fmt <- format(fm)
  expect_true(is.character(fmt))
  expect_true(is.matrix(fmt))
  expect_equal(dim(fmt), dim(m))

  m <- matrix(rnorm(4, 0.1), 2)
  fm <- formattable(m)
  fm[1, 1] <- m[1, 1] <- 0.1
  expect_true(all(fm == m))
})

test_that("formattable array", {
  m <- array(rnorm(60), c(3, 4, 5),
    list(letters[1:3], letters[1:4], letters[1:5]))
  fm <- formattable(m)
  expect_is(fm, c("formattable", "array"))
  expect_true(is.array(fm))
  expect_equal(dim(fm), dim(m))
  fmt <- format(fm)
  expect_true(is.character(fmt))
  expect_true(is.array(fmt))
  expect_equal(dim(fmt), dim(m))

  m <- array(rnorm(24, 0.1), c(2, 3, 4))
  fm <- formattable(m)
  fm[1, 1, 1] <- m[1, 1, 1] <- 0.1
  expect_true(all(fm == m))
})

test_that("is.formattable", {
  expect_equal(is.formattable(1), FALSE)
  expect_equal(is.formattable(formattable(1)), TRUE)
})

percent_preproc <- function(x) x * 100
percent_postproc <- function(str, x)
  paste0(str, ifelse(is.finite(x), "%", ""))
accounting_postproc <- function(str, x)
  sprintf(ifelse(is.na(x) | x >= 0, "%s", "(%s)"),
    gsub("-", "", str, fixed = TRUE))

#' Numeric vector with percentage representation
#'
#' @param x a numeric vector.
#' @param digits an integer to indicate the number of digits of the percentage string.
#' @param format format type passed to \code{\link{formatC}}.
#' @param ... additional parameters passed to \code{formattable}.
#' @export
percent <- function(x, digits, format = "f", ...)
  UseMethod("percent")

#' @rdname percent
#' @export
#' @examples
#' percent(rnorm(10, 0, 0.1))
#' percent(rnorm(10, 0, 0.1), digits = 0)
percent.default <- function(x, digits = 2L, format = "f", ...) {
  formattable(as_numeric(x), format = format, digits = digits, ...,
    preproc = "percent_preproc", postproc = "percent_postproc")
}

#' @rdname percent
#' @export
#' @examples
#' percent("0.5%")
#' percent(c("15.5%", "25.12%", "73.5"))
percent.character <- function(x, digits = NA, format = "f", ...) {
  valid <- grepl("^(.+)\\s*%$", x)
  pct <- gsub("^(.+)\\s*%$", "\\1", x)
  if (is.na(digits)) digits <- max(get_digits(x) - ifelse(valid, 0, 2))
  copy_dim(x, percent.default(as.numeric(pct) / ifelse(valid, 100, 1),
    digits = digits, format = "f"))
}

#' Numeric vector showing pre-specific digits
#'
#' @param x a numeric vector
#' @param digits an integer to indicate the number of digits to show.
#' @param format format type passed to \code{\link{formatC}}.
#' @param ... additional parameters passed to \code{formattable}.
#' @export
#' @examples
#' digits(pi, 2)
#' digits(123.45678, 3)
digits <- function(x, digits, format = "f", ...) {
  formattable(as.numeric(x), format = format, digits = digits, ...)
}

#' Numeric vector with thousands separators
#' @inheritParams percent
#' @param big.mark thousands separator
#' @export
comma <- function(x, digits, format = "f", big.mark = ",", ...)
  UseMethod("comma")

#' @rdname comma
#' @export
#' @examples
#' comma(1000000)
#' comma(c(1250000, 225000))
#' comma(c(1250000, 225000), format = "d")
comma.default <- function(x, digits = 2L, format = "f", big.mark = ",", ...) {
  formattable(as_numeric(x), format = format, big.mark = big.mark, digits = digits, ...)
}

#' @rdname comma
#' @export
#' @examples
#' comma("123,345.123")
comma.character <- function(x,
  digits = max(get_digits(x)), format = "f", big.mark = ",", ...) {
  copy_dim(x, comma.default(as.numeric(gsub(big.mark, "", x, fixed = TRUE)),
    digits = digits, format = format, big.mark = big.mark, ...))
}

#' Numeric vector with currency format
#' @inheritParams comma
#' @param symbol currency symbol
#' @param sep separator between symbol and value
#' @export
currency <- function(x, symbol, digits,
  format = "f", big.mark = ",", ...)
  UseMethod("currency")

#' @rdname currency
#' @export
#' @examples
#' currency(200000)
#' currency(200000, "\U20AC")
#' currency(1200000, "USD", sep = " ")
#' currency(1200000, "USD", format = "d", sep = " ")
currency.default <- function(x, symbol = "$",
  digits = 2L, format = "f", big.mark = ",", ..., sep = "") {
  x <- as_numeric(x)
  formattable(x, format = format, big.mark = big.mark, digits = digits, ...,
    postproc = function(str, x) sprintf("%s%s%s",
      ifelse(is.na(x), "", symbol), sep, str))
}

get_currency_symbol <- function(x) {
  sym <- unique(gsub("\\d|\\s|\\,|\\.", "", x))
  if (length(sym) > 1L) warning("Cannot find a unique symbol", call. = FALSE)
  sym[[1L]]
}

#' @rdname currency
#' @export
#' @examples
#' currency("$ 120,250.50")
#' currency("HK$ 120,250.50", symbol = "HK$")
#' currency("HK$ 120, 250.50")
currency.character <- function(x, symbol = get_currency_symbol(x),
  digits = max(get_digits(x)), format = "f", big.mark = ",", ...) {
  if (any(invalid <- !grepl("\\d", x)))
    warning("Invalid input in 'x': ", paste(x[invalid], collapse = ", "), call. = FALSE)
  num <- gsub("[^0-9\\.]", "", gsub(big.mark, "", x, fixed = TRUE))
  copy_dim(x, currency.default(as.numeric(num), symbol = symbol, digits = digits,
    format = format, big.mark = big.mark, ...))
}

#' Numeric vector with accounting format
#' @inheritParams comma
#' @export
accounting <- function(x, digits = 2L, format = "f", big.mark = ",", ...)
  UseMethod("accounting")

#' @rdname accounting
#' @export
#' @examples
#' accounting(15320)
#' accounting(-12500)
#' accounting(c(1200, -3500, 2600), format = "d")
accounting.default <- function(x, digits = 2L, format = "f", big.mark = ",", ...) {
  formattable(as_numeric(x), format = format, big.mark = big.mark, digits = digits, ...,
    postproc = "accounting_postproc")
}

#' @rdname accounting
#' @export
#' @examples
#' accounting(c("123,23.50", "(123.243)"))
accounting.character <- function(x, digits = max(get_digits(x)),
  format = "f", big.mark = ",", ...) {
  sgn <- ifelse(grepl("\\(.+\\)", x), -1, 1)
  num <- gsub("\\((.+)\\)", "\\1", gsub(big.mark, "", x, fixed = TRUE))
  copy_dim(x, accounting.default(sgn * as.numeric(num), digits = digits,
    format = format, big.mark = big.mark, ...))
}

#' Numeric vector with scientific format
#' @param x a numeric vector.
#' @param format format type passed to \code{\link{formatC}}.
#' @param ... additional parameter passed to \code{formattable}.
#' @export
#' @examples
#' scientific(1250000)
#' scientific(1253421, digits = 8)
#' scientific(1253421, digits = 8, format = "E")
scientific <- function(x, format = c("e", "E"), ...) {
  formattable(as_numeric(x), format = match.arg(format), ...)
}

#' Formattable object with prefix
#' @param x an object
#' @param prefix a character vector put in front of each non-missing
#' value in \code{x} as being formatted.
#' @param sep separator
#' @param ... additional parameter passed to \code{formattable}.
#' @param na.text text for missing values in \code{x}.
#' @export
#' @examples
#' prefix(1:10, "A")
#' prefix(1:10, "Choice", sep = " ")
#' prefix(c(1:10, NA), prefix = "A", na.text = "(missing)")
#' prefix(rnorm(10, 10), "*", format = "d")
#' prefix(percent(c(0.1,0.25)), ">")
prefix <- function(x, prefix = "", sep = "", ..., na.text = NULL) {
  formattable(x, ...,
    postproc = list(function(str, x)
      paste0(ifelse(xna <- is.na(x), "", paste0(prefix, sep)),
        if (is.null(na.text)) str else ifelse(xna, na.text, str))))
}

#' Formattable object with suffix
#' @param x an object
#' @param suffix a character vector put behind each non-missing
#' value in \code{x} as being formatted.
#' @param sep separator
#' @param ... additional parameter passed to \code{formattable}.
#' @param na.text text for missing values in \code{x}.
#' @export
#' @examples
#' suffix(1:10, "px")
#' suffix(1:10, ifelse(1:10 >= 2, "units", "unit"), sep = " ")
#' suffix(c(1:10, NA), "km/h", na.text = "(missing)")
#' suffix(percent(c(0.1, 0.25)), "*")
suffix <- function(x, suffix = "", sep = "", ..., na.text = NULL) {
  formattable(x, ...,
    postproc = list(function(str, x) {
      xna <- is.na(x)
      paste0(if (is.null(na.text)) str else ifelse(xna, na.text, str),
        ifelse(xna, "", paste0(sep, suffix)))
    }))
}

context("formats")

test_that("percent", {
  expect_identical(format(percent(numeric())), character())

  data <- c(-0.05, 0.15, 0.252, 0.3003)
  obj <- percent(data)
  expect_is(obj, c("formattable", "numeric"))
  expect_equal(format(obj), c("-5.00%", "15.00%", "25.20%", "30.03%"))
  expect_equal(format(percent(data, digits = 0)), c("-5%", "15%", "25%", "30%"))
  expect_equal(format(percent(obj, digits = 0)), c("-5%", "15%", "25%", "30%"))
  expect_warning(percent("a"), regexp = "NA")
  expect_equal(percent("1.00%"), percent(0.01))
  expect_equal(percent("1%"), percent(0.01, digits = 0L))
  expect_equal(percent(c("1.00%", "1%")), percent(c(0.01, 0.01)))
  expect_equal(percent(c("1.00%", "1.5")), percent(c(0.01, 1.5)))
  expect_is(percent(NA), "numeric")

  # percent matrix
  dim(data) <- c(2, 2)
  obj <- percent(data)
  expect_is(obj, c("formattable", "matrix"))
  expect_equal(format(obj), matrix(c("-5.00%", "15.00%", "25.20%", "30.03%"), 2))

  # parse percent from matrix
  x <- matrix(c("0.5%", "1.5%", "10.2%", "3.8%"), 2,
    dimnames = list(c("a", "b"), c("c", "d")))
  obj <- percent(x)
  expect_is(obj, c("formattable", "matrix"))
  expect_is(format(obj), "matrix")
  expect_equal(as.numeric(gsub("%", "", x, fixed = TRUE)) / 100, as.numeric(obj))
})

test_that("digits", {
  expect_identical(format(digits(numeric(), 1)), character())

  data <- c(pi, 123, 12.3456)
  obj <- digits(data, 1)
  expect_is(obj, c("formattable", "numeric"))
  expect_equal(format(obj), formatC(data, format = "f", digits = 1))
  expect_equal(format(digits(data, 2)), formatC(data, format = "f", digits = 2))
  expect_is(digits(NA, 2), "numeric")
})

test_that("comma", {
  expect_identical(format(comma(numeric())), character())

  data <- c(-5300, 10500, 20300, 35010)
  obj <- comma(data)
  expect_is(obj, c("formattable", "numeric"))
  expect_equal(format(obj), c("-5,300.00", "10,500.00", "20,300.00", "35,010.00"))
  expect_equal(format(comma(data, digits = 0)), c("-5,300", "10,500", "20,300", "35,010"))
  expect_equal(format(comma(obj, digits = 0, big.mark = "/")),
    c("-5/300", "10/500", "20/300", "35/010"))
  expect_warning(comma("a"))
  expect_equal(comma("123,234.56"), comma(123234.56))
  expect_equal(comma(c("1.23", "1,233.1232")), comma(c(1.23, 1233.1232), digits = 4))
  expect_is(comma(NA), "numeric")
})

test_that("currency", {
  expect_identical(format(currency(numeric())), character())

  data <- c(-5300, 10500, 20300, 35010)
  obj <- currency(data)
  expect_is(obj, c("formattable", "numeric"))
  expect_equal(format(obj, symbol = "$"), c("$-5,300.00", "$10,500.00", "$20,300.00", "$35,010.00"))
  expect_equal(format(currency(data, symbol = "$", digits = 0)), c("$-5,300", "$10,500", "$20,300", "$35,010"))
  expect_equal(format(currency(obj, symbol = "$", digits = 0, big.mark = "/")),
    c("$-5/300", "$10/500", "$20/300", "$35/010"))
  expect_equal(format(currency(1000, "USD", digits = 0, sep = " ")), "USD 1,000")
  expect_equal(format(currency("$ 123,234.50", symbol = "$", sep = " ")), "$ 123,234.50")
  expect_warning(currency("a"))
  expect_equal(format(currency("$ 123,234.50")), format(currency(123234.50, symbol = "$", digits = 2)))
  expect_equal(format(currency(c("$ 123,234.50", "$123.503"), digits = 3)),
    format(currency(c(123234.500, 123.503), symbol = "$", digits = 3)))
  expect_equal(format(currency(c("HK$ 123,234.50", "HK$ 123.503"), symbol = "HK$", digits = 3)),
    format(currency(c(123234.500, 123.503), symbol = "HK$", digits = 3)))
  expect_equal(format(currency(c("HK$ 123,234.50", "HK$ 123.503"), digits = 3)),
    format(currency(c(123234.500, 123.503), symbol = "HK$", digits = 3)))
  expect_is(currency(NA), "numeric")
})

test_that("accounting", {
  expect_identical(format(accounting(numeric())), character())

  data <- c(-5300, 10500, 20300, 35010)
  obj <- accounting(data)
  expect_is(obj, c("formattable", "numeric"))
  expect_equal(format(obj), c("(5,300.00)", "10,500.00", "20,300.00", "35,010.00"))
  expect_equal(format(accounting(data, digits = 0)), c("(5,300)", "10,500", "20,300", "35,010"))
  expect_equal(format(accounting(obj, digits = 0, big.mark = "/")),
    c("(5/300)", "10/500", "20/300", "35/010"))
  expect_warning(accounting("a"))
  expect_equal(accounting(c("123,23.50", "(123.243)")),
    accounting(c(12323.5, -123.243), digits = 3))
  expect_is(accounting(NA), "numeric")
})

test_that("scientific", {
  expect_identical(format(scientific(numeric())), character())

  data <- c(-5300, 10500, 20300, 35010)
  obj <- scientific(data)
  expect_is(obj, c("formattable", "numeric"))
  expect_equal(format(scientific(data, format = "e", digits = 2)),
    c("-5.30e+03", "1.05e+04", "2.03e+04", "3.50e+04"))
  expect_equal(format(scientific(data, format = "E", digits = 2)),
    c("-5.30E+03", "1.05E+04", "2.03E+04", "3.50E+04"))
  expect_warning(scientific("a"), "NA")
  expect_is(scientific(NA), "numeric")
})

test_that("prefix", {
  expect_identical(format(prefix(numeric())), character())

  data <- 1:10
  obj <- prefix(data)
  expect_is(obj, c("formattable", "integer"))
  expect_equal(format(prefix(data, "Item", sep = " ")),
    paste("Item", data))
  expect_equal(format(prefix(c(data, NA), "Item", sep = " ", na.text = "missing")),
    c(paste("Item", data), "missing"))

  data <- rnorm(10)
  expect_equal(format(prefix(data, "?", format = "f", digits = 2L)),
    paste0("?", formatC(data, format = "f", digits = 2L)))
  expect_equal(format(prefix(percent(data), ">")),
    paste0(">", format(percent(data))))
  expect_equal(format(prefix(percent(data), ">") + 0.1),
    paste0(">", format(percent(data) + 0.1)))
})

test_that("suffix", {
  expect_identical(format(suffix(numeric())), character())

  data <- 1:10
  obj <- suffix(data)
  expect_is(obj, c("formattable", "integer"))
  expect_equal(format(suffix(data, "px")), paste0(data, "px"))
  expect_equal(format(suffix(c(data, NA), "km/h", sep = " ", na.text = "missing")),
    c(paste(data, "km/h"), "missing"))

  data <- rnorm(10)
  expect_equal(format(suffix(data, "?", format = "f", digits = 2L)),
    paste0(formatC(data, format = "f", digits = 2L), "?"))
  expect_equal(format(suffix(percent(data), ">")),
    paste0(format(percent(data)), ">"))
  expect_equal(format(suffix(percent(data), ">") + 0.1),
    paste0(format(percent(data) + 0.1), ">"))
})

---
title: "Introduction to formattable package"
author: "Kun Ren"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to formattable package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo=FALSE, include=FALSE}
set.seed(123)
```

This package is designed for applying formatting on vectors and data frames to make data presentation easier, richer, more flexible and hopefully convey more information.

Atomic vectors are fundamental data structures in R. Some data can be read more easily with formatting. A numeric vector, for example, stores percentage numbers but is printed as typical floating numbers. This package provides functions to create data structures with predefined formatting rules so that these objects store the original data but are printed with formatting.

Several typical formattable numeric vectors are provided such as `percent`, `comma`, `currency`, `accounting` and `scientific`. These functions basically create numeric vectors with pre-defined formatting rules and parameters. For example,

```{r}
library(formattable)
p <- percent(c(0.1, 0.02, 0.03, 0.12))
p
```

The percent vector is no different from a numeric vector but has a percentage representation when printed. It works with arithmetic operations and other common functions and preserves its formatting.

```{r}
p + 0.05
p + percent(0.02)
p * 1.1
```

```{r}
max(p)
mean(p)
```

It also works with subsetting and sub-assignment:

```{r}
p[1:3]
p[[2]]
```

```{r}
p[[3]] <- 0.05
p
```

```{r}
balance <- accounting(c(1000, 500, 200, -150, 0, 1200))
balance
balance + 1000
```

These functions are specialized applications of what `formattable()` is designed to do. `formattable()` applies customizable formatting functions to objects of a wide range of classes like `numeric`, `logical`, `factor`, `Date`, `data.frame`, etc. 

When applied to `Date`, `formattable()` uses `format.Date()` as the default formatter function. The following code creates a `formattable Date` vector that is printed in the format of `%Y%m%d`. However, it is not a plain integer or character vector but of `Date` class and still allows date calculations. 

```{r}
dates <- formattable(as.Date(c("2016-05-01", "2016-05-10")), format = "%Y%m%d")
dates
dates + 30
```

When applied to a logical vector, we can customize how `TRUE` and `FALSE` values are printed.

```{r}
lv <- formattable(c(TRUE, FALSE, FALSE, TRUE), "yes", "no")
lv
!lv
```

Note that `isTRUE()` does not directly work with values of `lv` because `isTRUE()` uses `identical(x, TRUE)` and `lv[[1]]`, as a formattable logical value is not identical to a plain `TRUE`.

```{r}
lv[[1]]
isTRUE(lv[[1]])
```

If `isTRUE()` has to be applied, `lv == TRUE` returns a plain logical vector and works with `isTRUE()`. Other vectorized logical functions directly work with formattable logical vector with the formatting preserved.

```{r}
all(lv)
any(lv)
```

All formattable functions work with matrices and arrays.

```{r}
pm <- matrix(rnorm(6, 0.8, 0.1), 2, 3, 
  dimnames = list(c("a", "b"), c("X", "Y", "Z")))
pm
```

```{r}
fpm <- percent(pm)
fpm
```

```{r}
fpm["a", c("Y", "Z")]
```

```{r}
pa <- array(rnorm(12, 0.8, 0.1), c(2, 3, 2))
pa
```

```{r}
percent(pa)
```

When the formattable vectors are used as columns of a data frame, the formatting of each column is well preserved. A typical data frame may look more friendly with `formattable` column vectors. For example,

```{r}
p <- data.frame(
  id = c(1, 2, 3, 4, 5), 
  name = c("A1", "A2", "B1", "B2", "C1"),
  balance = accounting(c(52500, 36150, 25000, 18300, 7600), format = "d"),
  growth = percent(c(0.3, 0.3, 0.1, 0.15, 0.15), format = "d"),
  ready = formattable(c(TRUE, TRUE, FALSE, FALSE, TRUE), "yes", "no"))
p
```

The subset of a data frame also preserves the formatting of each column:

```{r}
p[1:3, c("name", "balance", "growth")]
```

---
title: "Formattable data frame"
author: "Kun Ren"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Formattable data frame}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo=FALSE, include=FALSE}
set.seed(123)
```

Formattable data frames are data frames to be rendered as HTML table with formatter functions applied, which resembles conditional formatting in Microsoft Excel. 

## Simple examples

Suppose we have the following data frame:

```{r}
scores <- data.frame(id = 1:5,
  prev_score = c(10, 8, 6, 8, 8),
  cur_score = c(8, 9, 7, 8, 9),
  change = c(-2, 1, 1, 0, 1))
```

In the console, it is printed as plain texts:

```{r}
scores
```

Using `knitr::kable()` or `formattable()`, the data frame can be rendered as HTML table which looks more friendly.

```{r}
library(formattable)
formattable(scores)
```

In fact, `formattable()` calls `knitr::kable()` internally to translate data frame to HTML code. In addition, `formattable()` supports formatter functions to customize the transformation between values in the data frame to HTML code to generate.

```{r}
plain_formatter <- formatter("span")
plain_formatter(c(1, 2, 3))
```

```{r}
width_formatter <- formatter("span",
  style = x ~ style(width = suffix(x, "px")))
width_formatter(c(10, 11, 12))
```

The values of `change` can be positive, negative or zero. We can make positives green, negatives red, and zeros black by creating a formatter function that performs conditional transformation from value to HTML code.

```{r}
sign_formatter <- formatter("span", 
  style = x ~ style(color = ifelse(x > 0, "green", 
    ifelse(x < 0, "red", "black"))))
sign_formatter(c(-1, 0, 1))
```

Note that we don't have to write HTML but use helper functions like `style()` and `ifelse()` to make it easier to specify conditions. Then we call `formattable()` on the data frame with a list of formatter functions so as to apply conditional formatting.

```{r}
formattable(scores, list(change = sign_formatter))
```

We can also create another formatter function that makes above-average values bold while leaving others unchanged.

```{r}
above_avg_bold <- formatter("span", 
  style = x ~ style("font-weight" = ifelse(x > mean(x), "bold", NA)))
formattable(scores, list(
  prev_score = above_avg_bold,
  cur_score = above_avg_bold,
  change = sign_formatter))
```

## Cross formatting

Sometimes, we need to format one column based on the values of another column. This can be easily done with one-sided formula in `formatter()`. When using `formatter("span", style = ~ expr)`, `expr` is evaluated in the data frame so that all columns are available for use.

```{r}
formattable(scores, list(
  cur_score = formatter("span", 
    style = ~ style(color = ifelse(change >= 0, "green", "red")))))
```

## Hiding columns

To hide columns, use `FALSE` formatter.

```{r}
formattable(scores, list(prev_score = FALSE))
```

## Using built-in formatters

To making formatting easier, formattable package provides a group of built-in formatter functions. Suppose we have the following data on a number of products. Some columns are already formattable vectors.

```{r}
products <- data.frame(id = 1:5, 
  price = c(10, 15, 12, 8, 9),
  rating = c(5, 4, 4, 3, 4),
  market_share = percent(c(0.1, 0.12, 0.05, 0.03, 0.14)),
  revenue = accounting(c(55000, 36400, 12000, -25000, 98100)),
  profit = accounting(c(25300, 11500, -8200, -46000, 65000)))
products
```

Without any formatter functions applied, the formattable data frame is directly rendered as an HTML table.

```{r}
formattable(products)
```

We can supply a list of formatter functions to make it look more colorful. For example, we apply `sign_formatter` to `profit` column so that values of different signs are displayed in different colors.

```{r}
formattable(products, list(profit = sign_formatter))
```

Using built-in functions like `color_tile()` and `color_bar()` makes it easier to compare the magnitute of values of specified columns.

```{r}
formattable(products, list(
  price = color_tile("transparent", "lightpink"),
  rating = color_bar("lightgreen"),
  market_share = color_bar("lightblue"),
  revenue = sign_formatter,
  profit = sign_formatter))
```

## Area formatting

Sometimes, it is useful to apply a formatter function to an area so that all cells in the area share one benchmark. Area formatting is supported through the syntax of `area(row, col) ~ formatter` in the formatter list.

The following example renders the three columns altogether so that they share the same benchmark, not independently.

```{r}
set.seed(123)
df <- data.frame(id = 1:10, 
  a = rnorm(10), b = rnorm(10), c = rnorm(10))
formattable(df, list(area(col = a:c) ~ color_tile("transparent", "pink")))
```

If a one-sided formula is supplied, the function will be applied to all cells.

```{r}
formattable(df[, -1], list(~ percent))
```

# Dynamically generating formatters

Since `formattable()` accepts a list of formatter functions, the list can be dynamically generated. For example, the following code applies row-wise formatting, that is, each row is colored independently.

```{r}
df <- cbind(data.frame(id = 1:10), 
  do.call(cbind, lapply(1:8, function(x) rnorm(10))))
formattable(df, lapply(1:nrow(df), function(row) {
  area(row, col = -1) ~ color_tile("lightpink", "lightblue")
}))
```

# Converting to `DT::datatables`

`as.datatable()` is designed to convert a formattable data frame to `DT::datatables`.

```{r, screenshot.force = FALSE}
as.datatable(formattable(products))
```

Some formatters can be preserved well after the conversion.

```{r, screenshot.force = FALSE}
as.datatable(formattable(products, list(
  price = color_tile("transparent", "lightpink"),
  revenue = sign_formatter,
  profit = sign_formatter)))
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formats.R
\name{percent}
\alias{percent}
\alias{percent.default}
\alias{percent.character}
\title{Numeric vector with percentage representation}
\usage{
percent(x, digits, format = "f", ...)

\method{percent}{default}(x, digits = 2L, format = "f", ...)

\method{percent}{character}(x, digits = NA, format = "f", ...)
}
\arguments{
\item{x}{a numeric vector.}

\item{digits}{an integer to indicate the number of digits of the percentage string.}

\item{format}{format type passed to \code{\link{formatC}}.}

\item{...}{additional parameters passed to \code{formattable}.}
}
\description{
Numeric vector with percentage representation
}
\examples{
percent(rnorm(10, 0, 0.1))
percent(rnorm(10, 0, 0.1), digits = 0)
percent("0.5\%")
percent(c("15.5\%", "25.12\%", "73.5"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formattable.R
\name{formattable.data.frame}
\alias{formattable.data.frame}
\title{Create a formattable data frame}
\usage{
\method{formattable}{data.frame}(
  x,
  ...,
  formatter = "format_table",
  preproc = NULL,
  postproc = NULL
)
}
\arguments{
\item{x}{a \code{data.frame}}

\item{...}{arguments to be passed to \code{formatter}.}

\item{formatter}{formatting function, \code{format_table} in default.}

\item{preproc}{pre-processor function that prepares \code{x} for
formatting function.}

\item{postproc}{post-processor function that transforms formatted
output for printing.}
}
\value{
a \code{formattable data.frame}
}
\description{
This function creates a formattable data frame by attaching
column or area formatters to the data frame. Each time the data frame
is printed or converted to string representation, the formatter
function will use the formatter functions to generate
formatted cells.
}
\details{
The formattable data frame is a data frame with lazy-bindings
of prespecified column formatters or area formatters.
The formatters will not be applied until the data frame is
printed to console or in a dynamic document. If the formatter
function has no side effect, the formattable data frame will
not be changed even if the formatters are applied to produce
the printed version.
}
\examples{
# mtcars (mpg in red)
formattable(mtcars,
   list(mpg = formatter("span", style = "color:red")))

# mtcars (mpg in red if greater than median)
formattable(mtcars, list(mpg = formatter("span",
   style = function(x) ifelse(x > median(x), "color:red", NA))))

# mtcars (mpg in red if greater than median, using formula)
formattable(mtcars, list(mpg = formatter("span",
   style = x ~ ifelse(x > median(x), "color:red", NA))))

# mtcars (mpg in gradient: the higher, the redder)
formattable(mtcars, list(mpg = formatter("span",
   style = x ~ style(color = rgb(x/max(x), 0, 0)))))

# mtcars (mpg background in gradient: the higher, the redder)
formattable(mtcars, list(mpg = formatter("span",
   style = x ~ style(display = "block",
   "border-radius" = "4px",
   "padding-right" = "4px",
   color = "white",
   "background-color" = rgb(x/max(x), 0, 0)))))

# mtcars (mpg in red if vs == 1 and am == 1)
formattable(mtcars, list(mpg = formatter("span",
    style = ~ style(color = ifelse(vs == 1 & am == 1, "red", NA)))))

# hide columns
formattable(mtcars, list(mpg = FALSE, cyl = FALSE))

# area formatting
formattable(mtcars, list(area(col = vs:carb) ~ formatter("span",
  style = x ~ style(color = ifelse(x > 0, "red", NA)))))

df <- data.frame(a = rnorm(10), b = rnorm(10), c = rnorm(10))
formattable(df, list(area() ~ color_tile("transparent", "lightgray")))
formattable(df, list(area(1:5) ~ color_tile("transparent", "lightgray")))
formattable(df, list(area(1:5) ~ color_tile("transparent", "lightgray"),
  area(6:10) ~ color_tile("transparent", "lightpink")))
}
\seealso{
\link{format_table}, \link{area}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formats.R
\name{digits}
\alias{digits}
\title{Numeric vector showing pre-specific digits}
\usage{
digits(x, digits, format = "f", ...)
}
\arguments{
\item{x}{a numeric vector}

\item{digits}{an integer to indicate the number of digits to show.}

\item{format}{format type passed to \code{\link{formatC}}.}

\item{...}{additional parameters passed to \code{formattable}.}
}
\description{
Numeric vector showing pre-specific digits
}
\examples{
digits(pi, 2)
digits(123.45678, 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatter.R
\name{proportion_bar}
\alias{proportion_bar}
\title{Create a color-bar formatter using proportion}
\usage{
proportion_bar(color = "lightgray", ...)
}
\arguments{
\item{color}{the background color of the bars}

\item{...}{additional parameters passed to \code{proportion}}
}
\description{
Create a color-bar formatter using proportion
}
\examples{
formattable(mtcars, list(mpg = proportion_bar()))
}
\seealso{
\link{color_bar}, \link{proportion}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formattable.R
\name{is.formattable}
\alias{is.formattable}
\title{Test for objects of 'formattable' class}
\usage{
is.formattable(x)
}
\arguments{
\item{x}{an object}
}
\value{
\code{TRUE} if \code{x} has class 'formattable';
\code{FALSE} otherwise.
}
\description{
Test for objects of 'formattable' class
}
\examples{
is.formattable(10)
is.formattable(formattable(10))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatter.R
\name{color_text}
\alias{color_text}
\title{Create a color-text formatter}
\usage{
color_text(...)
}
\arguments{
\item{...}{parameters passed to \code{gradient}.}
}
\description{
Create a color-text formatter
}
\examples{
formattable(mtcars, list(mpg = color_text("black", "red")))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/render.R
\name{as.htmlwidget.formattable}
\alias{as.htmlwidget.formattable}
\title{Convert formattable to an htmlwidget}
\usage{
\method{as.htmlwidget}{formattable}(x, width = "100\%", height = NULL, ...)
}
\arguments{
\item{x}{a \code{formattable} object to convert}

\item{width}{a valid \code{CSS} width}

\item{height}{a valid \code{CSS} height}

\item{...}{reserved for more parameters}
}
\value{
a \code{htmlwidget} object
}
\description{
formattable was originally designed to work in \code{rmarkdown} environments.
Conversion of a formattable to a htmlwidget will allow use in other contexts
such as console, RStudio Viewer, and Shiny.
}
\examples{
\dontrun{
library(formattable)
# mtcars (mpg background in gradient: the higher, the redder)
as.htmlwidget(
  formattable(mtcars, list(mpg = formatter("span",
   style = x ~ style(display = "block",
   "border-radius" = "4px",
   "padding-right" = "4px",
   color = "white",
   "background-color" = rgb(x/max(x), 0, 0))))
  )
)

# since an htmlwidget, composes well with other tags
library(htmltools)

browsable(
  tagList(
    tags$div( class="jumbotron"
              ,tags$h1( class = "text-center"
                        ,tags$span(class = "glyphicon glyphicon-fire")
                        ,"experimental as.htmlwidget at work"
              )
    )
    ,tags$div( class = "row"
               ,tags$div( class = "col-sm-2"
                          ,tags$p(class="bg-primary", "Hi, I am formattable htmlwidget.")
               )
               ,tags$div( class = "col-sm-6"
                          ,as.htmlwidget( formattable( mtcars ) )
               )
    )
  )
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formats.R
\name{currency}
\alias{currency}
\alias{currency.default}
\alias{currency.character}
\title{Numeric vector with currency format}
\usage{
currency(x, symbol, digits, format = "f", big.mark = ",", ...)

\method{currency}{default}(
  x,
  symbol = "$",
  digits = 2L,
  format = "f",
  big.mark = ",",
  ...,
  sep = ""
)

\method{currency}{character}(
  x,
  symbol = get_currency_symbol(x),
  digits = max(get_digits(x)),
  format = "f",
  big.mark = ",",
  ...
)
}
\arguments{
\item{x}{a numeric vector.}

\item{symbol}{currency symbol}

\item{digits}{an integer to indicate the number of digits of the percentage string.}

\item{format}{format type passed to \code{\link{formatC}}.}

\item{big.mark}{thousands separator}

\item{...}{additional parameters passed to \code{formattable}.}

\item{sep}{separator between symbol and value}
}
\description{
Numeric vector with currency format
}
\examples{
currency(200000)
currency(200000, "\U20AC")
currency(1200000, "USD", sep = " ")
currency(1200000, "USD", format = "d", sep = " ")
currency("$ 120,250.50")
currency("HK$ 120,250.50", symbol = "HK$")
currency("HK$ 120, 250.50")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform.R
\name{qrank}
\alias{qrank}
\title{Quantile ranks of a vector}
\usage{
qrank(x, ...)
}
\arguments{
\item{x}{a vector}

\item{...}{additional parameters passed to \code{rank}}
}
\description{
The quantile rank of a number in a vector is the relative
position of ranking resulted from rank divided by the length
of vector.
}
\examples{
qrank(mtcars$mpg)
}
\seealso{
\code{\link{rank}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formattable.R
\name{formattable.default}
\alias{formattable.default}
\title{Create a formattable object}
\usage{
\method{formattable}{default}(x, ..., formatter, preproc = NULL, postproc = NULL)
}
\arguments{
\item{x}{an object.}

\item{...}{arguments to be passed to \code{formatter}.}

\item{formatter}{formatting function, \code{formatC} in default.}

\item{preproc}{pre-processor function that prepares \code{x} for
formatting function.}

\item{postproc}{post-processor function that transforms formatted
output for printing.}
}
\value{
a \code{formattable} object that inherits from the original
object.
}
\description{
Create a formattable object
}
\examples{
formattable(rnorm(10), formatter = "formatC", digits = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatter.R
\name{color_bar}
\alias{color_bar}
\title{Create a color-bar formatter}
\usage{
color_bar(color = "lightgray", fun = "proportion", ...)
}
\arguments{
\item{color}{the background color of the bars}

\item{fun}{the transform function that maps the input vector to
values from 0 to 1. Uses \code{proportion} by default.}

\item{...}{additional parameters passed to \code{fun}}
}
\description{
Create a color-bar formatter
}
\examples{
formattable(mtcars, list(mpg = color_bar("lightgray", proportion)))
}
\seealso{
\link{normalize_bar}, \link{proportion_bar}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formats.R
\name{scientific}
\alias{scientific}
\title{Numeric vector with scientific format}
\usage{
scientific(x, format = c("e", "E"), ...)
}
\arguments{
\item{x}{a numeric vector.}

\item{format}{format type passed to \code{\link{formatC}}.}

\item{...}{additional parameter passed to \code{formattable}.}
}
\description{
Numeric vector with scientific format
}
\examples{
scientific(1250000)
scientific(1253421, digits = 8)
scientific(1253421, digits = 8, format = "E")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formats.R
\name{comma}
\alias{comma}
\alias{comma.default}
\alias{comma.character}
\title{Numeric vector with thousands separators}
\usage{
comma(x, digits, format = "f", big.mark = ",", ...)

\method{comma}{default}(x, digits = 2L, format = "f", big.mark = ",", ...)

\method{comma}{character}(x, digits = max(get_digits(x)), format = "f", big.mark = ",", ...)
}
\arguments{
\item{x}{a numeric vector.}

\item{digits}{an integer to indicate the number of digits of the percentage string.}

\item{format}{format type passed to \code{\link{formatC}}.}

\item{big.mark}{thousands separator}

\item{...}{additional parameters passed to \code{formattable}.}
}
\description{
Numeric vector with thousands separators
}
\examples{
comma(1000000)
comma(c(1250000, 225000))
comma(c(1250000, 225000), format = "d")
comma("123,345.123")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform.R
\name{normalize}
\alias{normalize}
\title{Normalize a vector to fit zero-to-one scale}
\usage{
normalize(x, min = 0, max = 1, na.rm = FALSE)
}
\arguments{
\item{x}{a numeric vector}

\item{min}{numeric value. The lower bound of the interval to normalize \code{x}.}

\item{max}{numeric value. The upper bound of the interval to normalize \code{x}.}

\item{na.rm}{a logical indicating whether missing values should be removed}
}
\description{
Normalize a vector to fit zero-to-one scale
}
\examples{
normalize(mtcars$mpg)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formattable.R
\name{formattable}
\alias{formattable}
\title{Generic function to create formattable object}
\usage{
formattable(x, ...)
}
\arguments{
\item{x}{an object.}

\item{...}{arguments to be passed to methods.}
}
\value{
a \code{formattable} object
}
\description{
This function is a generic function to create \code{formattable}
object, i.e. an object to which a formatting function and
related attribute are attached. The object works as ordinary object
yet has specially defined behavior as being printed or converted to
a string representation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.datatable.R
\name{as.datatable.formattable}
\alias{as.datatable.formattable}
\title{Convert \code{formattable} to a \code{\link[DT]{datatable}} htmlwidget}
\usage{
\method{as.datatable}{formattable}(x, escape = FALSE, ...)
}
\arguments{
\item{x}{a \code{formattable} object to convert}

\item{escape}{\code{logical} to escape \code{HTML}. The default is
\code{FALSE} since it is expected that \code{formatters} from
\code{formattable} will produce \code{HTML} tags.}

\item{...}{additional arguments passed to to \code{\link[DT]{datatable}}}
}
\value{
a \code{\link[DT]{datatable}} object
}
\description{
Convert \code{formattable} to a \code{\link[DT]{datatable}} htmlwidget
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/style.R
\name{style}
\alias{style}
\title{Create a string-representation of CSS style}
\usage{
style(...)
}
\arguments{
\item{...}{style attributes in form of \code{name = value}. Many CSS properties
contains \code{'-'} in the middle of their names. In this case, use
\code{"the-name" = value} instead. \code{NA} will cancel the attribute.}
}
\value{
a string-representation of css styles
}
\description{
Most HTML elements can be stylized by a set of CSS style
properties. This function helps build CSS strings using
conventional argument-passing in R.
}
\details{
The general usage of CSS styling is

\code{<span style = "color: red; border: 1px">Text</span>}

The text color can be specified by `color`, the border of
element by `border`, and etc.

Basic styles like \code{color}, \code{border}, \code{background}
work properly and mostly consistently in modern web browsers.
However, some style properties may not work consistently in
different browsers.
}
\examples{
style(color = "red")
style(color = "red", "font-weight" = "bold")
style("background-color" = "gray", "border-radius" = "4px")
style("padding-right" = "2px")

formattable(mtcars, list(
  mpg = formatter("span",
    style = x ~ style(color = ifelse(x > median(x), "red", NA)))))
}
\seealso{
\href{https://www.w3.org/Style/CSS/all-properties}{List of CSS properties},
  \href{https://www.w3schools.com/cssref/}{CSS Reference}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform.R
\name{vmap}
\alias{vmap}
\title{Vectorized map from element to case by index or string value}
\usage{
vmap(EXPR, ..., SIMPLIFY = TRUE)
}
\arguments{
\item{EXPR}{an expression evaluated to be character or numeric vector/list.}

\item{...}{The list of alternatives for each \code{switch}.}

\item{SIMPLIFY}{\code{TRUE} to simplify the resulted list to vector, matrix
or array if possible.}
}
\description{
This function is a vectorized version of \code{switch}, that is, for
each element of input vector, \code{switch} is evaluated and the results are
combined.
}
\examples{
x <- c("normal","normal","error","unknown","unknown")
vmap(x, normal = 0, error = -1, unknown = -2)

x <- c(1,1,2,1,2,2,1,1,2)
vmap(x, "type-A", "type-B")
}
\seealso{
\code{\link{switch}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formattable.R
\name{formattable.POSIXct}
\alias{formattable.POSIXct}
\title{Create a formattable POSIXct vector}
\usage{
\method{formattable}{POSIXct}(
  x,
  ...,
  formatter = "format.POSIXct",
  preproc = NULL,
  postproc = NULL
)
}
\arguments{
\item{x}{a vector of class \code{POSIXct}.}

\item{...}{arguments to be passed to \code{formatter}.}

\item{formatter}{formatting function, \code{format.POSIXct} in default.}

\item{preproc}{pre-processor function that prepares \code{x} for
formatting function.}

\item{postproc}{post-processor function that transforms formatted
output for printing.}
}
\value{
a \code{formattable} POSIXct vector
}
\description{
Create a formattable POSIXct vector
}
\examples{
times <- as.POSIXct("2015-04-10 09:30:15") + 1:5
ftimes <- formattable(times, format = "\%Y\%m\%dT\%H\%M\%S")
ftimes
ftimes + 30
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formattable.R
\name{formattable.factor}
\alias{formattable.factor}
\title{Create a formattable factor object}
\usage{
\method{formattable}{factor}(x, ..., formatter = "vmap", preproc = NULL, postproc = NULL)
}
\arguments{
\item{x}{a factor object.}

\item{...}{arguments to be passed to \code{formatter}.}

\item{formatter}{formatting function, \code{vmap} in default.}

\item{preproc}{pre-processor function that prepares \code{x} for
formatting function.}

\item{postproc}{post-processor function that transforms formatted
output for printing.}
}
\value{
a \code{formattable} factor object.
}
\description{
Create a formattable factor object
}
\examples{
formattable(as.factor(c("a", "b", "b", "c")),
  a = "good", b = "fair", c = "bad")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatter.R
\name{color_tile}
\alias{color_tile}
\title{Create a color-tile formatter}
\usage{
color_tile(...)
}
\arguments{
\item{...}{parameters passed to \code{gradient}.}
}
\description{
Create a color-tile formatter
}
\examples{
formattable(mtcars, list(mpg = color_tile("white", "pink")))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\docType{package}
\name{formattable-package}
\alias{formattable-package}
\title{The formattable package}
\description{
This package is designed for applying formatting on vectors and
data frames to make data presentation easier, richer, more flexible
and hopefully convey more information.
}
\details{
Atomic vectors are basic units to store data. Some data can be read more easily
with formatting. A numeric vector, for example, stores a group of
percentage numbers yet still shows in the form of typical floating numbers. This
package provides functions to create data structures with predefined
formatting rules so that these objects stores the original data but are printed
with formatting.

On the other hand, in a typical workflow of dynamic document production, \code{knitr}
and \code{rmarkdown} are powerful tools to render documents with R code to different
types of portable documents.

\code{knitr} package is able to render a RMarkdown document (markdown document
with R code chunks to be executed sequentially) to Markdown document.
\code{rmarkdown} calls \code{pandoc} to render markdown document to HTML web page.
To put a table from a \code{data.frame} on the page, one may call \code{knitr::kable}
to produce its markdown representation. By default the resulted table is in a plain
theme with no additional formatting. However, in some cases, additional formatting
may help clarify the information and make contrast of the data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/render.R
\name{renderFormattable}
\alias{renderFormattable}
\title{Widget render function for use in Shiny}
\usage{
renderFormattable(expr, env = parent.frame(), quoted = FALSE)
}
\arguments{
\item{expr}{an expression that generates a valid \code{formattable} object}

\item{env}{the environment in which to evaluate expr.}

\item{quoted}{is expr a quoted expression (with quote())?
This is useful if you want to save an expression in a variable.}
}
\description{
Widget render function for use in Shiny
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/render.R
\name{as.htmlwidget}
\alias{as.htmlwidget}
\title{Generic function to create an htmlwidget}
\usage{
as.htmlwidget(x, ...)
}
\arguments{
\item{x}{an object.}

\item{...}{arguments to be passed to methods.}
}
\value{
a \code{htmlwidget} object
}
\description{
This function is a generic function to create an \code{htmlwidget}
to allow HTML/JS from R in multiple contexts.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatter.R
\name{formatter}
\alias{formatter}
\title{Create a formatter function making HTML elements}
\usage{
formatter(.tag, ...)
}
\arguments{
\item{.tag}{HTML tag name. Uses \code{span} by default.}

\item{...}{functions to create attributes of HTML element from data colums.
The unnamed element will serve as the function to produce the inner text of the
element. If no unnamed element is provided, \code{identity} function will be used
to preserve the string representation of the colum values. Function and formula are
accepted. See details for how different forms of formula will behave differently.}
}
\value{
a function that transforms a column of data (usually an atomic vector)
to formatted data represented in HTML and CSS.
}
\description{
Create a formatter function making HTML elements
}
\details{
This function creates a \code{formatter} object which is essentially a
closure taking a value and optionally the dataset behind.

The formatter produces a character vector of HTML elements represented
as strings. The tag name of the elements are specified by \code{.tag},
and its attributes are calculated with the given functions or formulas
specified in \code{...} given the input vector and/or dataset in behind.

Formula like \code{x ~ expr} will behave like \code{function(x) expr}.
Formula like \code{~expr} will be evaluated in different manner: \code{expr}
will be evaluated in the data frame with the enclosing environment being
the formula environment. If a column is formatted according to multiple
other columns, \code{~expr} should be used and the column names can directly
appear in \code{expr}.
}
\examples{
top10red <- formatter("span",
  style = x ~ ifelse(rank(-x) <= 10, "color:red", NA))
yesno <- function(x) ifelse(x, "yes", "no")
formattable(mtcars, list(mpg = top10red, qsec = top10red, am = yesno))

# format one column by other two columns
# make cyl red for records with both mpg and disp rank <= 20
f1 <- formatter("span",
  style = ~ ifelse(rank(-mpg) <= 20 & rank(-disp) <= 20, "color:red", NA))
formattable(mtcars, list(cyl = f1))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/style.R
\name{gradient}
\alias{gradient}
\title{Create a matrix from vector to represent colors in gradient}
\usage{
gradient(x, min.color, max.color, alpha = NULL, use.names = TRUE, na.rm = TRUE)
}
\arguments{
\item{x}{a numeric vector.}

\item{min.color}{color of minimum value.}

\item{max.color}{color of maximum value.}

\item{alpha}{logical of whether to include alpha channel. \code{NULL}
to let the function decide by input.}

\item{use.names}{logical of whether to preserve names of input vector.}

\item{na.rm}{logical indicating whether to ignore missing values as \code{x}
is normalized. (defult is \code{TRUE})}
}
\value{
a matrix with rgba columns in which each row corresponds to the rgba
value (0-255) of each element in input vector \code{x}. Use \code{csscolor}
to convert the matrix to css color strings compatible with web browsers.
}
\description{
Create a matrix from vector to represent colors in gradient
}
\examples{
gradient(c(1,2,3,4,5), "white", "red")
gradient(c(5,4,3,2,1), "white", "red")
gradient(c(1,3,2,4,5), "white", "red")
gradient(c(1,3,2,4,5), rgb(0,0,0,0.5), rgb(0,0,0,1), alpha = TRUE)
}
\seealso{
\code{\link{csscolor}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formattable.R
\name{formattable.Date}
\alias{formattable.Date}
\title{Create a formattable Date vector}
\usage{
\method{formattable}{Date}(x, ..., formatter = "format.Date", preproc = NULL, postproc = NULL)
}
\arguments{
\item{x}{a vector of class \code{Date}.}

\item{...}{arguments to be passed to \code{formatter}.}

\item{formatter}{formatting function, \code{format.Date} in default.}

\item{preproc}{pre-processor function that prepares \code{x} for
formatting function.}

\item{postproc}{post-processor function that transforms formatted
output for printing.}
}
\value{
a \code{formattable} Date vector
}
\description{
Create a formattable Date vector
}
\examples{
dates <- as.Date("2015-04-10") + 1:5
fdates <- formattable(dates, format = "\%m/\%d/\%Y")
fdates
fdates + 30
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform.R
\name{proportion}
\alias{proportion}
\title{Rescale a vector relative to the maximal absolute value in the vector}
\usage{
proportion(x, na.rm = FALSE)
}
\arguments{
\item{x}{a numeric vector}

\item{na.rm}{a logical indicating whether missing values should be removed}
}
\description{
Rescale a vector relative to the maximal absolute value in the vector
}
\examples{
proportion(mtcars$mpg)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatter.R
\name{normalize_bar}
\alias{normalize_bar}
\title{Create a color-bar formatter using normalize}
\usage{
normalize_bar(color = "lightgray", ...)
}
\arguments{
\item{color}{the background color of the bars}

\item{...}{additional parameters passed to \code{normalize}}
}
\description{
Create a color-bar formatter using normalize
}
\examples{
formattable(mtcars, list(mpg = normalize_bar()))
}
\seealso{
\link{color_bar}, \link{normalize}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formattable.R
\name{format_table}
\alias{format_table}
\title{Format a data frame with formatter functions}
\usage{
format_table(
  x,
  formatters = list(),
  format = c("html", "markdown", "pandoc"),
  align = "r",
  ...,
  digits = getOption("digits"),
  table.attr = "class=\\"table table-condensed\\""
)
}
\arguments{
\item{x}{a \code{data.frame}.}

\item{formatters}{a list of formatter functions or formulas.
The existing columns of \code{x} will be applied the formatter
function in \code{formatters} if it exists.

If a formatter is specified by formula, then the formula will be
interpreted as a lambda expression with its left-hand side being
a symbol and right-hand side being the expression using the symbol
to represent the column values. The formula expression will be evaluated
in the environment of the formula.

If a formatter is \code{FALSE}, then the corresponding column will be hidden.

Area formatter is specified in the form of
\code{area(row, col) ~ formatter} without specifying the column name.}

\item{format}{The output format: html, markdown or pandoc?}

\item{align}{The alignment of columns: a character vector consisting
of \code{'l'} (left), \code{'c'} (center), and/or \code{'r'} (right).
By default, all columns are right-aligned.}

\item{...}{additional parameters to be passed to \code{knitr::kable}.}

\item{digits}{The number of significant digits to be used for numeric
and complex values.}

\item{table.attr}{The HTML class of \code{<table>} created when
\code{format = "html"}}
}
\value{
a \code{knitr_kable} object whose \code{print} method generates a
string-representation of \code{data} formatted by \code{formatter} in
specific \code{format}.
}
\description{
This is an table generator that specializes in creating
formatted table presented in HTML by default.
To generate a formatted table, columns or areas of the
input data frame can be transformed by formatter functions.
}
\examples{
# mtcars (mpg in red)
format_table(mtcars,
   list(mpg = formatter("span", style = "color:red")))

# mtcars (mpg in red if greater than median)
format_table(mtcars, list(mpg = formatter("span",
   style = function(x) ifelse(x > median(x), "color:red", NA))))

# mtcars (mpg in red if greater than median, using formula)
format_table(mtcars, list(mpg = formatter("span",
   style = x ~ ifelse(x > median(x), "color:red", NA))))

# mtcars (mpg in gradient: the higher, the redder)
format_table(mtcars, list(mpg = formatter("span",
   style = x ~ style(color = rgb(x/max(x), 0, 0)))))

# mtcars (mpg background in gradient: the higher, the redder)
format_table(mtcars, list(mpg = formatter("span",
   style = x ~ style(display = "block",
   "border-radius" = "4px",
   "padding-right" = "4px",
   color = "white",
   "background-color" = rgb(x/max(x), 0, 0)))))

# mtcars (mpg in red if vs == 1 and am == 1)
format_table(mtcars, list(mpg = formatter("span",
    style = ~ style(color = ifelse(vs == 1 & am == 1, "red", NA)))))

# hide columns
format_table(mtcars, list(mpg = FALSE, cyl = FALSE))

# area formatting
format_table(mtcars, list(area(col = vs:carb) ~ formatter("span",
  style = x ~ style(color = ifelse(x > 0, "red", NA)))))

df <- data.frame(a = rnorm(10), b = rnorm(10), c = rnorm(10))
format_table(df, list(area() ~ color_tile("transparent", "lightgray")))
format_table(df, list(area(1:5) ~ color_tile("transparent", "lightgray")))
format_table(df, list(area(1:5) ~ color_tile("transparent", "lightgray"),
  area(6:10) ~ color_tile("transparent", "lightpink")))
}
\seealso{
\link{formattable}, \link{area}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formattable.R
\name{formattable.numeric}
\alias{formattable.numeric}
\title{Create a formattable numeric vector}
\usage{
\method{formattable}{numeric}(x, ..., formatter = "formatC", preproc = NULL, postproc = NULL)
}
\arguments{
\item{x}{a numeric vector.}

\item{...}{arguments to be passed to \code{formatter}.}

\item{formatter}{formatting function, \link{formatC} in default.}

\item{preproc}{pre-processor function that prepares \code{x} for
formatting function.}

\item{postproc}{post-processor function that transforms formatted
output for printing.}
}
\value{
a \code{formattable} numeric vector.
}
\description{
Create a formattable numeric vector
}
\examples{
formattable(rnorm(10), format = "f", digits = 1)
formattable(rnorm(10), format = "f",
  flag="+", digits = 1)
formattable(1:10,
  postproc = function(str, x) paste0(str, "px"))
formattable(1:10,
  postproc = function(str, x)
    paste(str, ifelse(x <= 1, "unit", "units")))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/style.R
\name{csscolor}
\alias{csscolor}
\title{Generate CSS-compatible color strings}
\usage{
csscolor(x, format = c("auto", "hex", "rgb", "rgba"), use.names = TRUE)
}
\arguments{
\item{x}{color input}

\item{format}{the output format of color strings}

\item{use.names}{logical of whether to preserve the names of input}
}
\value{
a character vector of CSS-compatible color strings
}
\description{
Generate CSS-compatible color strings
}
\examples{
csscolor(rgb(0, 0.5, 0.5))
csscolor(c(rgb(0, 0.2, 0.2), rgb(0, 0.5, 0.2)))
csscolor(rgb(0, 0.5, 0.5, 0.2))
csscolor(gradient(c(1,2,3,4,5), "white", "red"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/style.R
\name{icontext}
\alias{icontext}
\title{Create icon-text elements}
\usage{
icontext(
  icon,
  text = list(NULL),
  ...,
  simplify = TRUE,
  provider = getOption("formattable.icon.provider", "glyphicon"),
  class_template = getOption("formattable.icon.class_template",
    "{provider} {provider}-{icon}")
)
}
\arguments{
\item{icon}{a character vector or list of character vectors
of icon names.}

\item{text}{a character vector of contents.}

\item{...}{additional parameters (reserved)}

\item{simplify}{logical to indicating whether to return
the only element if a single-valued list is resulted.}

\item{provider}{the provider of icon set.}

\item{class_template}{a character value to specifiy to template of the class
with \code{"{provider}"} to represent \code{provider} value and \code{"{icon}"} to
represent \code{icon} values.}
}
\description{
Create icon-text elements
}
\examples{
icontext("plus")
icontext(c("star","star-empty"))
icontext(ifelse(mtcars$mpg > mean(mtcars$mpg), "plus", "minus"), mtcars$mpg)
icontext(list(rep("star",3), rep("star",2)), c("item 1", "item 2"))
}
\seealso{
\href{https://getbootstrap.com/docs/3.4/components/#glyphicons}{Glyphicons in Bootstrap},
\href{https://glyphicons.com/}{Glyphicons}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/render.R
\name{formattableOutput}
\alias{formattableOutput}
\title{Widget output function for use in Shiny}
\usage{
formattableOutput(outputId, width = "100\%", height = "0")
}
\arguments{
\item{outputId}{output variable to read from}

\item{width}{a valid \code{CSS} width or a number}

\item{height}{valid \code{CSS} height or a number}
}
\description{
Widget output function for use in Shiny
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatter.R
\name{area}
\alias{area}
\title{Create an area to apply formatter}
\usage{
area(row, col)
}
\arguments{
\item{row}{an expression of row range. If missing,
\code{TRUE} is used instead.}

\item{col}{an expression of column range. If missing,
\code{TRUE} is used instead.}
}
\description{
Create an representation of two-dimenstional area
to apply formatter function. The area can be one or
more columns, one or more rows, or an area of rows
and columns.
}
\details{
The function creates an \code{area} object to store
the representation of row and column selector expressions.
When the function is called, the expressions and environment
of \code{row} and \code{column} are captured for
\code{format_table} to evaluate within the context of the
input \code{data.frame}, that is, \code{rownames} and
\code{colnames} are defined in the context to be the indices
of rows and columns, respectively. Therefore, the row names
and column names are avaiable symbols when \code{row}
and \code{col} are evaluated, respectively, which makes it
easier to specify range with names, for example,
\code{area(row = row1:row10, col = col1:col5)}.
}
\examples{
area(col = c("mpg", "cyl"))
area(col = mpg:cyl)
area(row = 1)
area(row = 1:10, col = 5:10)
area(1:10, col1:col5)
}
\seealso{
\link{format_table}, \link{formattable.data.frame}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formattable.R
\name{formattable.logical}
\alias{formattable.logical}
\title{Create a formattable logical vector}
\usage{
\method{formattable}{logical}(x, ..., formatter = "ifelse", preproc = NULL, postproc = NULL)
}
\arguments{
\item{x}{a logical vector.}

\item{...}{arguments to be passed to \code{formatter}.}

\item{formatter}{formatting function, \code{formattable::ifelse} in default.}

\item{preproc}{pre-processor function that prepares \code{x} for
formatting function.}

\item{postproc}{post-processor function that transforms formatted
output for printing.}
}
\value{
a \code{formattable} logical vector.
}
\description{
Create a formattable logical vector
}
\examples{
logi <- c(TRUE, TRUE, FALSE)
flogi <- formattable(logi, "yes", "no")
flogi
!flogi
any(flogi)
all(flogi)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formats.R
\name{accounting}
\alias{accounting}
\alias{accounting.default}
\alias{accounting.character}
\title{Numeric vector with accounting format}
\usage{
accounting(x, digits = 2L, format = "f", big.mark = ",", ...)

\method{accounting}{default}(x, digits = 2L, format = "f", big.mark = ",", ...)

\method{accounting}{character}(x, digits = max(get_digits(x)), format = "f", big.mark = ",", ...)
}
\arguments{
\item{x}{a numeric vector.}

\item{digits}{an integer to indicate the number of digits of the percentage string.}

\item{format}{format type passed to \code{\link{formatC}}.}

\item{big.mark}{thousands separator}

\item{...}{additional parameters passed to \code{formattable}.}
}
\description{
Numeric vector with accounting format
}
\examples{
accounting(15320)
accounting(-12500)
accounting(c(1200, -3500, 2600), format = "d")
accounting(c("123,23.50", "(123.243)"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formats.R
\name{prefix}
\alias{prefix}
\title{Formattable object with prefix}
\usage{
prefix(x, prefix = "", sep = "", ..., na.text = NULL)
}
\arguments{
\item{x}{an object}

\item{prefix}{a character vector put in front of each non-missing
value in \code{x} as being formatted.}

\item{sep}{separator}

\item{...}{additional parameter passed to \code{formattable}.}

\item{na.text}{text for missing values in \code{x}.}
}
\description{
Formattable object with prefix
}
\examples{
prefix(1:10, "A")
prefix(1:10, "Choice", sep = " ")
prefix(c(1:10, NA), prefix = "A", na.text = "(missing)")
prefix(rnorm(10, 10), "*", format = "d")
prefix(percent(c(0.1,0.25)), ">")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formats.R
\name{suffix}
\alias{suffix}
\title{Formattable object with suffix}
\usage{
suffix(x, suffix = "", sep = "", ..., na.text = NULL)
}
\arguments{
\item{x}{an object}

\item{suffix}{a character vector put behind each non-missing
value in \code{x} as being formatted.}

\item{sep}{separator}

\item{...}{additional parameter passed to \code{formattable}.}

\item{na.text}{text for missing values in \code{x}.}
}
\description{
Formattable object with suffix
}
\examples{
suffix(1:10, "px")
suffix(1:10, ifelse(1:10 >= 2, "units", "unit"), sep = " ")
suffix(c(1:10, NA), "km/h", na.text = "(missing)")
suffix(percent(c(0.1, 0.25)), "*")
}
